% 2021 10 29:   the first time the theorist gave us a wrong matrix...
% so this is attempt 2, now we have triple checked the matrix, so hopefully
% it will go better this time :)
% 
% so changes in experiment:
%   1.  again less pump power (to prevent detector blinding)
%           I realized that the detector blinding is a problem with
%           relative detection efficiency characterization
%   2.  more integration time (to average out noise)
%   3.  use spectral filters (12 nm FWHM) to slightly decrease spectral
%       width and to improve the fidelities by a bit :))
%   4.  measure the entire 12x12 matrix
%   5.  also test for incoupling efficiencies
%   6.  measure 
% 
% 

% units for input: (unless specified)
% distance / position       mm
% time                      ms
% speed                     nm /s
% acceleration              micron /s /s
% triggerlevel              volt
%
clc
close all
clearvars -except detect smrct c pd spec shut
clear settings measure

% mode 0:   scan with constant velocity
% mode 1:   with well defined steps & integration time
% mode 2:   go in/out HOM dip   (obsolete!)
% Trigger:  0   no trigger diode
%           1   a trigger diode is used (To minimize dark counts)
settings.mode = 1;
settings.Trigger = 0;


% meta data selection
% you want to get the laser power & spectrum or not?
% only makes sense in mode 1
settings.getSpectrum = 0;
settings.getPower = 1;


% channels & counter
% set the channels (2 or more)
% set the Channel of the trigger diode 
% set the trigger level (first: detectors, 2nd: trigger diode)
% set the coincidence window (in ps!)
% set external delay (Relative to hardcoded in detector) in ps
% settings.ChList    = [1 2 4:12 14];
% settings.ChTrigger = 15;
settings.ChList = [9:12, 6:8, 14];
settings.ChTrigger = [];
settings.TriggerLvl= [];            % dont use unless you know what you are doing
settings.CoinWindow= 1000;
settings.delays    = zeros(1,15);    % include that of trigger diode(s)!
settings.nFoldCoin = 2;


% delays for the chip characterization
settings.delays(1) =  200;
settings.delays(2) =  100;
settings.delays(4) =  500;
settings.delays(5) =  500;
settings.delays(6) = 1600;
settings.delays(7) = -100;
settings.delays(8) = 1150;  
settings.delays(9) = 0;     
settings.delays(10)= 0;     
settings.delays(11)= -100;  
settings.delays(12)= 100;   
settings.delays(14)= 0;     
settings.delays(15)= -300;  


% scan options stage (smaract)
% set the begin/ end position (-25 to 25)
% specify # scans (forwards + backwards = 2)
% and the stage channel
settings.posBegin = 9.0;
settings.posEnd   = 10;
settings.nSteps   = 2;              % only for mode 1
settings.tRepeat  = 1; 
settings.stageCh  = 0;


% parameters for mode 0: cont. scan
% the velocity
% the time interval between datapoints (ms)
settings.speed  = 0.03e6;
settings.waitTime = 100;   


% parameters for mode 1: step
% Set the integration time (how long to count the counts)
% The wait time (to guarantee it is at the right position)
% Set the stepsize between measurement points
% and the number of measurements per position
settings.intTime = 1;      % in seconds;
settings.waitTime= 100;
settings.nRepeat = 1;


% optional: custom position list
% leave empty or comment out if you dont want to use it
clear settings.customPosList
% settings.customPosList = sort([8.18000000000000 8.18800000000000 8.19600000000000 8.20400000000000 8.21200000000000 8.22000000000000 8.22800000000000 8.23600000000000 8.24400000000000 8.25200000000000 8.26000000000000 8.26800000000000 8.27600000000000 8.28400000000000 8.29200000000000 8.30000000000000 8.30800000000000 8.31600000000000 8.32400000000000 8.33200000000000 8.34000000000000 8.34800000000000 8.35600000000000 8.36400000000000 8.37200000000000 8.38000000000000 8.38800000000000 8.396 8.404 8.412 8.42, 8.302, 8.304, 8.305]);


% process the settings
[settings, measure] = processSettings( settings);	


%% === DON'T CHANGE ANYTHING BELOW === %
% ========== REALLY, DON'T! ========== %
% ==================================== %
% ======= ONLY RUN THESE PARTS ======= %
% ==================================== %

% start initializing all the equipment
% initialize the detector
if exist('detect', 'var') == 1
    % delete instance if it already exists
    detect.delete()
    clear detect
end
detect = initializeDetector( settings);

% initialize the smaract stage
if exist('smrct', 'var') == 0
     % delete instance if it already exists
    smrct = smaract.getInstance();
end
initializeStage( smrct, settings)


% initialize other devices
if exist( 'pd', 'var') == 0 
    % also check if you need it, otherwise: dummy variable
    if settings.getPower == 1
        pd = photodiode.getInstance();
    else 
        pd = [];
    end
end
if exist( 'spec', 'var') == 0 
    if settings.getSpectrum == 1
        spec = spectrometer_775.getInstance();
    else
        spec = [];
    end
end
% if exist( 'shut', 'var') == 0
%     shut = Shutters.getInstance();
%     shut = [];
% end




%% measure the QuantumLoopGravity matrices
clc
% select folder / filenames where you can find the matrix
Folder = 'C:\Users\COPSLAB_local\Documents\multi_photon_source\data\20210427_QuantumLoopGravity\20212116\Voltage settings\';
FileName = 'voltages_LQP_targetMatrix';


% the detection efficiency dataset
FolderF = 'C:\Users\COPSLAB_local\Documents\multi_photon_source\data\20210427_QuantumLoopGravity\20212116\Voltage settings\';
prefixF = 'characterization_Voltage_input_';
prefixFs= 'characterization_input_';
middleF = '_output_';
suffixF = '.txt';


% data saving options
% actual data
inputConfig = '01_08'
FolderS = 'C:\Users\COPSLAB_local\Documents\multi_photon_source\data\20210427_QuantumLoopGravity\20212116\';
suffixS  = 'QLG_Data_inputConfig_';
suffixSf = 'Trigger_QLG_Characterization_input_';
a = input('press Ctrl+C if you disagree with this input mode, or any numeric if you agree')



% select the list of position you want to measure at
% and set up the HOM measurement file + heaters (if applicable)
switch inputConfig
    case '01_02'
        HOMcentr= 9.1;
        
    case '01_03'
        HOMcentr = 11.51;
        
    case '01_04'
        HOMcentr = 10.251;
        
    case '01_05'
        HOMcentr = 7.423;
        
    case '01_06'
        HOMcentr = 10.718;
        
    case '01_07'
        HOMcentr = 9.849;
        
    case '01_08'
        HOMcentr = 9.083;
        
    case '01_09'
        HOMcentr = 11.685;
        
    case '01_10'
        HOMcentr = 9.458;
        
    case '01_11'
        HOMcentr = 8.013;
        
    case '01_12'
        HOMcentr = 10.834;
        
        % these should be a constant offset compared to '01_xx', matching
        % difference in fiber length of input modes 01 and 02
    case '02_03'
        HOMcentr = 10.983;
        
    case '02_04'
        HOMcentr = 9.721;
        
    case '02_05'
        HOMcentr = 6.893;
        
    case '02_06'
        HOMcentr = 10.199;
        
    case '02_07'
        HOMcentr = 9.322;
        
    case '02_08'
        HOMcentr = 8.556;
        
    case '02_09'
        HOMcentr = 11.158;
        
    case '02_10'
        HOMcentr = 8.931;
        
    case '02_11'
        HOMcentr = 7.486;
        
    case '02_12'
        HOMcentr = 10.307;
        
    otherwise
        warning('No HOM position / offset specified')
        HOMcentr = 10.0;
end
% offset: outside the HOM dip
offset = HOMcentr - 0.3;
        
offsetN = 2;
HOMn = 14;
HOMwidth = 0.16;    % 80 micron in each direction > max hom shift we expect
posList = [offset*ones(1,offsetN),...
    linspace(HOMcentr-HOMwidth/2, HOMcentr+HOMwidth/2, HOMn+1)];

% connect to the chip
c = chip.getInstance('2021_Quix_12x12_camel');
disp(' ')


% setup the detectors for the loss characterization / singles measurement
settings.ChList     = [1 2 4:12 14];
settings.ChTrigger  = 15;
settings.Trigger    = 1;
settings.intTime    = 10;
settings.nRepeat    = 6;
settings.nFoldCoin  = 1;
settings.customPosList = [];
clear settings.customPosList
settings.posBegin   = offset;
settings.posEnd     = offset;
settings.nSteps     = 1;
[settings, measure] = processSettings( settings);
detect.delete
detect = initializeDetector( settings);
detect.setDeadTime( 1:14, ones(1,14)*80);




% from all inputs to all outputs
if strcmp( inputConfig(1:2), '01')
    
    % if inputConfig = '01_02': measure all detection efficiencies from
    % input 2
    if strcmp( inputConfig(4:5), '02')
        shut.openShutter(1)     % only let S1 into the system, to input 1
        shut.closeShutter(2)
        for ii = 1 : 12
            % load in the new characterization matrix
            Fname = [prefixF, '1', middleF, num2str(ii)];
            FnameS= [prefixFs,'1', middleF, num2str(ii)];
            c.loadHeaterQuix( FolderF, [Fname, suffixF]);
            pause(1)
            c.open; c.measurement(1); c.close;
            
            % start measuring splitting ratio
            measure = doMeasurement( settings, measure, detect, smrct, pd, spec);
            
            % save the fiber data
            saveData( [FolderS,'Efficiency_Characterization\'],...
                [FnameS, '.mat'], settings, measure)
            disp(' ')
        end
    end
    
    
    % for all inputConfig = '01_xx', measure detection efficiencies from
    % input mode xx ( xx = [1 3 4 5 6 7 8 9 10 11 12])
    shut.openShutter(2)     % only let I1 into the system, to input xx
    shut.closeShutter(1)
    
    for ii = 1 : 12
        % load in the new characterization matrix
        Fname = [prefixF, num2str(str2num(inputConfig(4:5))), middleF, num2str(ii)];
        FnameS= [prefixFs,inputConfig(4:5), middleF, num2str(ii)];
        c.loadHeaterQuix( FolderF, [Fname, suffixF]);
        pause(1)
        c.open; c.measurement(1); c.close;
        
        % start measuring splitting ratio
        measure = doMeasurement( settings, measure, detect, smrct, pd, spec);
        
        % save the fiber data
        saveData( [FolderS,'Efficiency_Characterization\'],...
            [FnameS, '.mat'], settings, measure)
        disp(' ')
    end
        
end


disp('Loss characterization data measured')
disp(' ')
disp(' ')
shut.openShutter(1:2)


% setup the chip voor LQG matrix
c.loadHeaterQuix( Folder, [FileName,'.txt']);
pause(1)
c.open; T = c.measurement(1); c.close


% now measure all singles
if strcmp( inputConfig(1:2), '01')
    
    
    
    % measure the singles
    % send the signal to the first input
    shut.closeShutter(1)
    measure = doMeasurement( settings, measure, detect, smrct, pd, spec);
    saveData( [FolderS, 'Matrix'],...
        [suffixS, inputConfig, '_singles1'], settings, measure)
    
    % send the idler to the 2nd input
    shut.closeShutter(2)
    shut.openShutter(1)
    measure = doMeasurement( settings, measure, detect, smrct, pd, spec);
    saveData( [FolderS, 'Matrix'],...
        [suffixS, inputConfig, '_singles2'], settings, measure)
    shut.openShutter(1:2)
    

    disp('Finished measuring the singles')
else
    disp('No need to measure the singles for this input configuration')
end
disp(' ')


% do the HOM dip scan
shut.openShutter(1:2)


% setup the detectors for 2 photon HOM scan
% general settings (default HOM measurement)
settings.ChList     = [1 2 4:12 14];
settings.ChTrigger  = [];
settings.Trigger    = 0;
settings.intTime    = 10;      % in seconds;
settings.nRepeat    = 10;
settings.nFoldCoin  = 2;
settings.customPosList = posList;
[settings, measure] = processSettings( settings);
detect.delete
pause(1)
detect = initializeDetector( settings);
detect.setDeadTime( 1:14, ones(1,14)*80);



%
% start the HOM measurement and save the data
measure = doMeasurement( settings, measure, detect, smrct, pd, spec);
saveData( [FolderS, 'Matrix'],...
    [suffixS, inputConfig, '_HOM'], settings, measure)




% turn off the heaters and close the shutters
disp('Measurements complete, close everything')
shut.closeShutter(1:2)
detect.delete







%% FUNCTIONS LIVE BELOW
% so you don't have to deal with the ugly code ;)
function [settings, measure] = processSettings( settings)
% processes the given input settings 


% mode = 0      -->     cont. scan
if settings.mode == 0
    settings.nRepeat = 1;
	settings.nSteps  = 2;
    settings.inttime = 100;
	%settings.waitTime = ?;
	settings.tRepeat = 1;
    
    measure.posList = zeros(1, settings.tRepeat);
    for ii = 1 : settings.tRepeat
        if mod(ii,2) == 1
            measure.posList(ii) = settings.posEnd;
        else
            measure.posList(ii) = settings.posBegin;
        end
    end
        
% mode = 1      -->     predefined steps
elseif settings.mode == 1
    % processes if there is a customPosList or not
    if isfield( settings, 'customPosList') && ~isempty(settings.customPosList)
        disp('custom position list found')
        settings.posBegin = settings.customPosList(1);
        settings.posEnd   = settings.customPosList(end);
        settings.nSteps   = length( settings.customPosList);
        
        % set up memory allocation / posList
        settings.posList1 = settings.customPosList;
    else
       
        settings.posList1 = linspace( settings.posBegin, settings.posEnd,...
            settings.nSteps);

    end
    
    % initialize measurement settings / where to dump data
    measure.posList = fixPosList( settings);
    
    
    % set last important specs
    if length(settings.posList1) > 1
        stepSize = abs(settings.posList1(2)-settings.posList1(1));
    else
        stepSize = 0.1;     % dummy value, in case no movement is needed
    end
    settings.speed        = stepSize *4e6;
    settings.acceleration = stepSize * 4e3; 
end


% get the coincidence channels and the strings of all channels
measure.singles = [];
measure.coincidences = [];
settings.coinStr = [];
settings.singStr = [];
settings = getChannels( settings);


% general settings
settings.colors = lines(max(settings.nCoin,settings.nCh));


% structure to save all singles in
for ii = 1 : settings.nCh
	ChName = settings.singStr{ii};
	measure.singles.(ChName)  = zeros( settings.nRepeat, settings.nSteps*settings.tRepeat);
end
% the same, now for coincidences
for ii = 1 : settings.nCoin
	ChName = settings.coinStr{ii};
	measure.coincidences.(ChName)  = zeros( settings.nRepeat, settings.nSteps*settings.tRepeat);
end
end

function measure = measureMode1( settings, measure, detect, smrct)
% Performs the measurement model1 (measure at fixed positions for a fixed
% duration of time) for the specified settings. Write down the results in
% the measure structure
% also makes a figure of the data

% wipe possible previous data
for ii = 1:settings.nCh
	measure.singles.(settings.singStr{ii}) = zeros(size(measure.singles.(settings.singStr{ii})));
end
for ii = 1:settings.nCoin
	measure.coincidences.(settings.coinStr{ii}) = zeros(size(measure.coincidences.(settings.coinStr{ii})));
end


% set up the figure
figure(1);
clf
hold on
xlabel('position (mm)')
ylabel('Counts (Hz)')
box on
if length( measure.posList) > 1
    xlim([ min(measure.posList), max(measure.posList)])
end

% button to stop the whole procedure
handle.Stop = 0;
button = uicontrol( 'Position', [20,20, 100,20],...
	'String', 'Stop', 'Callback', @(src,evnt)pushed(smrct,settings.stageCh) );
set( gcf, 'UserData',0);        % userdata = check if button is pressed

% set up the legend
for cc = 1: settings.nCoin
    plot( measure.posList(1), 0, '.',...
        'MarkerSize', 15, 'color', settings.colors(cc,:))
end
legend( settings.coinStr)

% start going back and forth
for ii = 1 : length(measure.posList)

	% make the next step, wait until it is done
	smrct.moveAbsolute( settings.stageCh, measure.posList(ii)*1e6, 0)
	while smrct.getStatus( settings.stageCh, 0) == 4
		pause(0.1)
	end
	
    
	% get the counts
    % repeat this for all # measurements you want at each step
	for kk = 1:settings.nRepeat
		
        % get the data
        cnt2 = detect.getCounts(1);
        
        % write down the data to the required struct.
		for cc = 1:settings.nCoin
			measure.coincidences.(settings.coinStr{cc})(kk,ii) = cnt2(settings.nCh+cc);
		end
		for ss = 1:settings.nCh
			measure.singles.(settings.singStr{ss})(kk, ii) = cnt2(ss);
        end
    end

	% update plot
	for cc = 1 : settings.nCoin
		plot( measure.posList(ii),...
            mean( measure.coincidences.(settings.coinStr{cc})(:, ii)),...
            '.', 'MarkerSize', 15, 'color', settings.colors(cc,:),...
            'HandleVisibility','off')
	end

	
	% prepare for the next step
	if get( gcf, 'UserData') == 1
		% if the stop button is pressed, then Userdata == 1
		% so in that case: stop
        
        % delete allocated space which wasnt used
        measure.posList(ii+1:end) = [];
        for cc = 1 : settings.nCoin
            measure.coincidences.(settings.coinStr{cc})(:, ii+1:end) = [];
        end
		break;
	end
end

disp('Finished')

end

function saveData( path, suffix, settings, measure)
% saves the data in a .txt file and a .mat file
% only works with mode 1, not mode 0
%
% note: .txt version is still to be implemented

% make sure that the folder path ends properly
if path(end) ~= '\'
    path = [path, '\'];
end

% build filename
prefix = datestr(now,'yyyymmdd');
fileName = [prefix, '_data_', suffix];

% store all data in a file
data.FileName = fileName;
data.Date = prefix;
data.Time = datestr(now,'HH:MM:SS');
data.DetectionChannels = settings.ChList;
data.CoincidenceWindow = settings.CoinWindow;
data.IntegrationTime = settings.intTime;
data.DetectorDelays = settings.delays;
data.StageChannel = settings.stageCh;
data.nDatapoints = size(measure.coincidences,1);
data.nPositions = size(measure.coincidences,2);
data.PositionList = measure.posList;
data.Singles = measure.singles;
data.Coincidences = measure.coincidences;
data.Pump.Power = measure.power;
data.Pump.Spectrum = measure.spectrum;

% optional data
if isfield( settings, 'ChTrigger')
    data.TriggerChannels = settings.ChTrigger;
else
    data.TriggerChannels = [];
end


% save as .mat
save( [path, fileName, '.mat'], 'data')

end

function plotErrorBar( settings, measure)
% plots all the data with errorbars

% sanity check, mode 0 does not store all datapoints (only the last one)
if settings.mode == 0
	error('cannot plot the data in mode 0')
end


% initialize figure
figure(2)
clf
subplot(2,1,1)
hold on
for cc = 1 : settings.nCoin
    cnt = measure.coincidences.(settings.coinStr{cc});
    errorbar( double(measure.posList), mean(cnt,1), std(cnt,0,1),...
        '.-', 'MarkerSize', 10, 'color', settings.colors(cc,:))
end
hold off
FancyPlot(2,2)


xlabel('Position (mm)')
ylabel('Coincidence Counts (Hz)')
legend( settings.coinStr)

ylim([0 max(ylim)])
box on

subplot(2,1,2)
hold on
for ss = 1 : settings.nCh
    cnt = measure.singles.(settings.singStr{ss});
    errorbar( double(measure.posList), mean( cnt,1), std( cnt,0,1),...
        '.-', 'MarkerSize', 10, 'color', settings.colors(ss,:));
end
FancyPlot(2,2)

xlabel('Position (mm)')
ylabel('Single Counts (Hz)')
legend(settings.singStr, 'Location', 'SouthEast')
ylim([0 max(ylim)])
box on
% % plot( xFit/1e6  , yFit)


end

function settings = getChannels( settings)
% get the list of all coincidence channels and the strings of these
% channels (also the strings of the single channels)

% against stupid input
if settings.nFoldCoin > size( settings.ChList,2)
    warning('Not enough coincidence detectors given (eg, a 4 fold coincidence with 3 detectors)')
end

% set coincidence channels
% check if there is a trigger diode present
% if so, add all trigger channel(s) to it (can be an array)
if settings.Trigger == 1
    nFixed = length( settings.ChTrigger) ;
    
    % verify that the trigger channels are not added already 
    % loop through all elements of ChList, verify for each if the element
    % corresponds with any of the Trigger channels: if so: save the index
    indDelete = [];
    for jj = 1 : length(settings.ChList)
        for ii = 1 : nFixed
            if settings.ChList(jj) == settings.ChTrigger(ii)
                indDelete = [indDelete, ii];
                break
            end
        end
    end
    % now delete them
    settings.ChTrigger(indDelete) = [];
    
else
    nFixed = 0;
    settings.ChTrigger = [];
end

% in case the user asks for more coincidence channels than available
if settings.nFoldCoin > length( settings.ChList)
    warning(['The requested ', numstr(settings.nFoldCoin),...
        '-fold coincidences is not possible as the total number of channels is ',...
        num2str( length(settings.ChList))])
    settings.nFoldCoin = length( settings.ChList);
end

% build up the list
settings.coinList = nchoosek( settings.ChList, settings.nFoldCoin);
nPermut           = size( settings.coinList,1);


% now append the fixed trigger channels to all measurement combinations
if ~isempty( settings.ChTrigger)
    settings.coinList = [settings.coinList, settings.ChTrigger...
                    .* ones(nPermut,nFixed)];
end

                
% Get number of singles and coincidences
% first add the fixed herald arm (not earlier since that would mess up the
% building of the coincidence list)
settings.ChList = [settings.ChList, settings.ChTrigger];
settings.nCh = size( settings.ChList,2);
settings.nCoin = size( settings.coinList,1);


% get the coincidence strings
settings.coinList2 = [];
settings.coinStr = [];
for cc = 1 : settings.nCoin
    settings.coinList2{cc} = settings.coinList(cc,:);
    % build up the string of arbitrary length 
    coinStr = [];
    for ii = 1 : length( settings.coinList2{cc})
        coinStr = [coinStr, 'C', num2str(settings.coinList(cc,ii))];
    end
    settings.coinStr{cc} = coinStr;
end

% get the singles strings
settings.singStr = [];
for ss = 1 : settings.nCh
    settings.singStr{ss} = ['C', num2str(settings.ChList(ss))];
end
end

function data = getSpectra( spec)
% get the spectrum of the spectrometer with the handle spec
% output: 
%   data    2xn array with 1st row the wavelengths (nm)
%                          2nd row the intensity (a.u.)

% get the wavelength bins and the intensity at each bin
wavelength = spec.getWavelengths();
spectrum   = spec.getSpectrum();

% combine the data
data = [wavelength; spectrum];

end

function power = getPower( pd)
% get the power from the (calibrated) reference photo diode

% get the power from the photodiode
[P1, P2] = pd.getPower();
power = [P1, P2];

% tell the user how much power we now have
disp(['Pump power per crystal is  = ', num2str(round(mean(power),1),'%05.1f'), ' mW'])
end

function measure = doMeasurement( settings, measure, detect, smrct, pd, spec)
% container function from which the exact subroutines will be launched

disp(['Start measurement at ', datestr( now, 'HH:MM:ss')])

% get current pump power and spectrum (if wanted)
if settings.getPower == 1
    measure.power = getPower( pd);
else
    measure.power = -1;
end
if settings.getSpectrum == 1
    measure.spectrum = getSpectra( spec);
else
    measure.spectrum = -1;
end


% select the correct mode
if settings.mode == 0
%     shut.open     open the shutters
	measureMode0( settings, measure, detect, smrct);

elseif settings.mode == 1
%     shut.open
	measure = measureMode1( settings, measure, detect, smrct);

else
	error('unkown measurement mode selected')
end


end

function measureMode0( settings, measure, detect, smrct)

% get initial datapoint (for legend)
pos = settings.posBegin;
cnt = detect.getCounts(1);
		
% set up the graph
figure(1)
clf
hold on
xlabel('position (mm)')
ylabel('Counts (Hz)')
box on
xlim([min(settings.posBegin,settings.posEnd) max(settings.posBegin,settings.posEnd)])

% plot the first datapoint
for jj = 1 : settings.nCoin
	plot( double(pos)/1e6, 0,...
		'.', 'MarkerSize', 15, 'Color', settings.colors(jj,:))
end
legend( settings.coinStr)
		

% button to stop the whole procedure
button = uicontrol( 'Position', [20,20, 100,20],...
	'String', 'Stop', 'Callback', @(src,evnt)pushed(smrct,settings.stageCh));

for ii = 1 : settings.tRepeat
	
	% start moving in the correct direction
	% and keep getting data until the stage stops moving
	smrct.moveAbsolute( settings.stageCh, measure.posList(ii)*1e6, 0);
	while smrct.getStatus( settings.stageCh, 0) == 4
		
		% get the positions and coincidences (don't store)
		ptemp = smrct.getPosition( settings.stageCh);
		cnt = detect.getCounts(0);
		
		% update plot
		for jj = 1 : settings.nCoin
			plot( double(ptemp)/1e6, cnt(settings.nCh+jj),...
				'.', 'MarkerSize', 15, 'Color', settings.colors(jj,:),...
                'HandleVisibility','off')
		end
		
		% wait the specified time
		pause( settings.waitTime/1e3)
	end
end
ylim([0 max(ylim)])
disp('Finished')
	
end

function posList = fixPosList( settings)
% sets up the correct order of the positions

nPoints = length( settings.posList1);
posList = repmat( settings.posList1, settings.tRepeat);

% every even one = going back. So reverse order there
for ii = 2 : 2 : settings.tRepeat
    index1 = (ii-1)*nPoints+1;
    index2 = index1+nPoints;
    posList( index1 : index2) = flip( settings.posList1);
end

end

function initializeStage( smrct, settings)
% sets the smaract stage at the correct starting position
% in    smaract handle and settings struct

% smaract stage
smrct.setSpeed( settings.stageCh, 1e6);
smrct.moveAbsolute( settings.stageCh, settings.posBegin*1E6, 0)
while smrct.getStatus( settings.stageCh,0) == 4
    pause(0.1)
end
pos = smrct.getPosition( settings.stageCh);

% set the speed and acceleration ready for the measurement
smrct.setSpeed( settings.stageCh, settings.speed);
if isfield( settings, 'acceleration')
    smrct.setAcceleration( settings.stageCh, settings.acceleration);
end

% tell the user that the stage is ready to go
disp( ['Stage ', num2str(settings.stageCh), ' at ', num2str(round(double(pos)/1e6,2)), ' mm'])
disp( ' ')

end

function detect = initializeDetector( settings)
% in    settings
% initialize the detector

detect = detectors.getInstance( [settings.ChList],...
    settings.coinList2);

% for all channels
% if a new Trigger level is provided
if isempty(settings.TriggerLvl) == 0
    for ii = 1 : settings.nCh
        detect.TimeTagger.setTriggerLevel( settings.ChList(ii),...
            settings.TriggerLvl(1));
    end
    % if triggermode is also there
    if settings.Trigger == 1
        for ii = 1 : length(settings.ChTrigger)
            detect.TimeTagger.setTriggerLevel( settings.ChTrigger(ii),...
                settings.TriggerLvl(ii));
        end 
    end
end
% in case you use the trigger diode: divide counts by 10 to
% prevent overflow (timetagger cannot handle 80 MHz pulses)
for ii = 1 : length( settings.ChList)
    if settings.ChList(ii) == 15
        warning('TriggerCh 15 = trigger diode, set count divisor')
        detect.TimeTagger.setEventDivider( 15, 12);
        detect.TimeTagger.setTriggerLevel( 15, -0.035);
    end
end
            
detect.setCoincidenceWindow( settings.CoinWindow);
detect.TimeTagger.clearOverflows
detect.setInputDelayExt( settings.delays);
if settings.intTime*1e12 < 1e9
    warning('Integration time is less than 1 ms, rebiasing will occur often')
end
detect.ExposureTime = settings.intTime*1e12;
end

function [V,C,Q] = getVisibility( xData, yData, nOffset, plotFit)
% gets the HOM visibility for the set xData and yData
% Input parameters
% 	xData 	array with positions 	(1,N) in mm
% 	yData 	array with coincidences (n,N) in Hz
% 	nOffset # datapoints outside the HOM dip (assumes first datapoints)
%   plotFit 1 if you want a plot of the Fit
%
% Output:
% 	V 		Visibility ( |V| <= 1)
%   optional:
%   C       Classical: average counts outside HOM dip/peak
%   Q       Quantum: average counts inside dip / peak

% average over multiple datapoints per position
if size( yData,1) > 1
    meanY = mean( yData);
else
    meanY = yData;
end

% get distinguishable photons
C = mean( meanY(1:nOffset));

% get the HOM dip/peak by fitting
% also: remove the offset datapoints as they are not important anyhow
[~,~,Q] = fitHOM( xData(nOffset+1:end), meanY(nOffset+1:end), C, plotFit);

% get the visibility
 V = (C - Q) / C;
 
 % sanity checks
 if V < -1
	V = -1;
elseif V > 1
	V = 1;
end
end

function [xFit, yFit, HOM] = fitHOM( xData, yData, C, plotFit)
% fits the HOM with a local polynomial around the dip/peak, returns array with fit data
% input
%   xData     	array with positions, (1,N) in mm!
%   yData     	array with coincidences, (1,N)
%	C 			average coincidences for dist. photons 
%   plotFit     1 if you want a plot of the fit
%
% output
%	xFit 		array with positions of the fit (mm)
%	yFit 		array with fitted coincidences 
% 	HOM 		minimum of maximum of HOM (number)

% average over all the datapoints at 1 position
if size(yData,1) > 1
    meanY = mean( yData);
else
    meanY = yData;
end

% find out if we have a dip or peak
% and set the initial conditions accordingly
testPeak = abs( max(meanY) - C);
testDip  = abs( min(meanY) - C);

% determine if peak/dip and find the index of the center datapoint in HOM
if testPeak > testDip
	% peak
	Amplitude = 1;
	ind = find( max(meanY) == meanY);
elseif testPeak < testDip
	% a dip
	Amplitude = -1;
    ind = find( min(meanY) == meanY);
end

nAround = 4;
indFit = ind-nAround/2 : ind+nAround/2;
% to do: protect against ind < 0 or > max index

% fit a polynomial through the minimum
ft = fit( xData(indFit)', meanY(indFit)', 'poly2' );
xFit = [min(xData(indFit)) : 0.0005 : max(xData(indFit))];
yFit = ft.p1*xFit.^2 + ft.p2*xFit.^1 + ft.p3; 

% extract the HOM dip or peak
if Amplitude == 1
	HOM = max( yFit);
else
	HOM = min( yFit);
end

% if wanted: plot the fit
if plotFit == 1
    figure; clf
    hold on
    plot( xFit, yFit)
    plot( xData, meanY, 'o')
    hold off
end
end

function pushed( smrct, stageCh)
% the button in the plots to stop the measurement
% input:  handle to the smaract stage
%         stage channel number of the one that needs to be stopped

% stop the stage
smrct.stopMoving( stageCh);
% stop mode 1 || 2 by setting UserData in the figure to 1
set(gcf, 'UserData', 1);
% set graph
gca;
ylim([0 max(ylim)])

end
