clc
clearvars -except HOMe orderHOMe tthigh
close all

% Change location to current folder
cwd = fileparts(matlab.desktop.editor.getActiveFilename);
cd(cwd)
addpath(cwd)
addpath(genpath(cwd))



% selection of data
% Folder    where experimental data is saved
% FolderU   where to find ideal matrices
% nameC     filename (incl. wildcard *) of HOM scans
% nameU     filename (incl. wildcard *) of target (=ideal) matrices 
Folder = [cwd, '\data\Matrix\'];
FolderU= [cwd, '\data\Target matrix\'];
nameU  = 'LQP_targetMatrix.txt';


% selection of which output-modes are used (=Swabian channel numbering)
outputModes = [1 2 4:12 14];
nOutput = length( outputModes);

% the same of input modes (based on chip numbering)
[inputModes, inputStr, inputStrS] = getInputModes( 12);
matStr = getMatrixString( 1);
nameC = '2021121*_data_QLG_Data_';


%% Get target data / ideal results 
% get the target matrices
matrices = 1;
% Ut = processTargetMatrix( );
% manually load tthigh from \..._attempt_2\: 
tthigh = load('tt-high.csv');
Ut.Matrix_001 = rephase(tthigh);
HOMt = getHOMTarget( Ut, inputModes);


%% Get Singles rates
clc
[Rates, Power, relCnts, Singles] = getRates( Folder, matrices, inputStrS, matStr);



%% Get the loss characterization
clc
close all

doPlotSingles  = 0;
doPlotRelative = 0;
relTrans{11} = zeros(1,12);

% old edition

FolderRef =  [cwd, '\data\Efficiency_oldway\'];
naam0 = '2021121*_data_characterization_input_';

ind = 1;
for ii = [1, 1 : 11]
    name    = [naam0, inputStrS{ii}, '_matrix*'];
    relTrans{ind} = getDetectionLosses(FolderRef, name, doPlotSingles, doPlotRelative);
    ind = ind+1;
end


% by manual check, I found this works reasonably well:
% inputLoss = [ 0.9934 0.9956 1.0175 1.0165 0.9947 0.9901 1.0076 0.9878 0.9245 1.0084 1.0077 1.0563];
inputLoss = [1.1465 1.0031 1.1341 0.9853 0.7769 0.9129 1.1242 0.9708 0.7992 1.0921 0.9617 1.0931];
% inputLoss = ones(1,12);
%% Corect the rates for the losses (as told by Quix) and the measured
% relative detection efficiencies
clc 
% close all
doNormalize = 1;
RatesC = correctRates( matrices, matStr, Rates, Power, relTrans, inputLoss, relCnts, doNormalize);
RatesC2= correctRates( matrices, matStr, Rates, Power, relTrans, inputLoss, relCnts, 0);
% plotSingles( RatesC, Ut, matrices, matStr, inputStr)

colSum = sum(RatesC.Matrix_001)
rowSum = sum(RatesC.Matrix_001')'
mean(rowSum)
std(rowSum)




%% get HOM visibilities
clc
nMoving   = 1;
nFit      = 1;
doPlotFit = 0;
doPlotC   = 0;
matrices  = 1;
[data, HOMe, HOMposition] = processCoincidences( Folder, nameC, matrices,...
                        inputStr, matStr, 1, doPlotC, doPlotFit, nMoving, nFit );
clc




%% Plot HOM positions
plotEach = 0;
plotTotal= 1;
matrices = 1 ;
plotHOMposition( HOMposition, data.Matrix_001, matrices, inputStr, matStr, plotEach, plotTotal)



%% fix the order in which the data is stored, change what is needed for Obrien
orderHOMe = fixOrderHOMs( HOMe, matrices, inputStr, matStr);
orderHOMt = fixOrderHOMs( HOMt, matrices, inputStr, matStr);
clc



%%
clc
[R1, F1, ~] = simObrien_v2( Ut.Matrix_001,  1, (abs(Ut.Matrix_001)), orderHOMt.Matrix_001, 1);
[R2, F2, ~] = simObrien_v2( Ut.Matrix_001,  1, (RatesC.Matrix_001),  orderHOMt.Matrix_001, 1);
[R3, F3, ~] = simObrien_v2( Ut.Matrix_001,  1, (RatesC2.Matrix_001), orderHOMt.Matrix_001, 1);
[R4, F4, Rdata] = simObrien_v2( Ut.Matrix_001,  1, (RatesC.Matrix_001), orderHOMe.Matrix_001, .9);

figure(3)
clf
subplot(1,2,1)
imagesc(abs(Ut.Matrix_001))
set(gca,'YDir','normal')
axis square
colorbar
title('Target')
FancyPlot(0)

subplot(1,2,2)
imagesc(abs(R4))
% imagesc(U)
set(gca,'YDir','normal')
axis square
colorbar
title('After unitarization')
FancyPlot(0)

modes1 = 1:8;
modes2 = 1:4;

R_opt = optimizeIOPhases(Ut.Matrix_001, R4)

F_amp = getFidelity( abs(Ut.Matrix_001), abs(R4))
F_pha = getFidelity( Ut.Matrix_001, R4)
F_pha_opt = getFidelity( Ut.Matrix_001, R_opt)
F_amp_4x8 = getFidelity( abs(Ut.Matrix_001(modes1,modes2)), abs(R4(modes1,modes2)))
F_pha_4x8 = getFidelity( Ut.Matrix_001(modes1,modes2), R4(modes1,modes2))
F_pha_4x8_opt = getFidelity( Ut.Matrix_001(modes1,modes2), R_opt(modes1,modes2))


% Uexp2 = non unitarized version
Uexp2 = sqrt( RatesC.Matrix_001) .* exp( 1i* angle(R4));

%Uexp = unitarized version
% Uexp = R4;
Uexp = R_opt;

% writetable(table(R4), 'Reconstructed_Unitary_Stitched.txt')
% writetable(table(Uexp2), 'Non_unitarized.csv')
%% compare counts with amplitude
figure(1)
clf
subplot(4,1,1)
imagesc(abs(Ut.Matrix_001).^2)
title('Target', 'fontweight', 'normal')

subplot(4,1,2)
imagesc(abs(Rates.Matrix_001))
title('Rates', 'fontweight', 'normal')

subplot(4,1,3)
imagesc(abs(RatesC2.Matrix_001))
title('Rates corrected for loss', 'fontweight', 'normal')

subplot(4,1,4)
imagesc(abs(RatesC.Matrix_001))
title('Normalized rates, corrected for loss', 'fontweight', 'normal')

for ii = 1 : 4
    subplot(4,1,ii)
    axis square
colorbar
set(gca,'YDir','normal')
end



%% make fancy figure for the paper
hmap(1:256,1) = linspace(0,1,256);
hmap(:,[2 3]) = 0.7; %brightness
huemap = hsv2rgb(hmap);

clc
f = figure(1);
f.Position = [500 300 500 500];
clf

scl = 1;

[ha, ax] = tight_subplot(2,2, scl.*[.1 .05], scl.*[.15 0.1], scl.*[.1 .1]);

% modes1 = 1:8;
% modes2 = 1:4;
modes1 = 1:12;
modes2 = 1:12;


barSize = .8;
cLabelOff = 0.75;
ttOffset  = .5;

axes(ha(1))
h{1} = bar3( abs(Uexp(modes1,modes2)),barSize);
view(2)
zlim([0 1])
tt = title('Experiment', 'fontweight', 'normal');
tt.Position(2) = tt.Position(2)-ttOffset;

colormap(gca,cividis)


axes(ha(2))
h{3} = bar3(abs(Ut.Matrix_001(modes1,modes2)),barSize);
zlim([0 1])
view(2)
originalSize1 = get(gca, 'Position');

% somehow the colorbar takes the 'data' of the x or y axis (=modes)
% however, when exporting to .pdf this gets fixed somehow?
c = colorbar;
c.Limits = [0 1];
c.Ticks  = [0 1];
c.TickLabels = {'0', '1'};
c.Label.String = 'Amplitude';
c.Label.Position(1) = c.Label.Position(1) -cLabelOff ;
savePos = c.Label.Position(1);

set(gca, 'Position', originalSize1); 
tt = title('Target', 'fontweight', 'normal');
tt.Position(2) = tt.Position(2)-ttOffset;
colormap(gca,cividis);


axes(ha(3))
h{2} = bar3(angle(Uexp(modes1,modes2))+pi,barSize);
zlim([0 2*pi])
set( gca, 'ztick', [0 pi 2*pi], 'zticklabel', {'-\pi', '0', '\pi'}, 'TickLabelInterpreter', 'tex')
view(2)
colormap(gca,twilight)


axes(ha(4))
h{4} = bar3(angle(Ut.Matrix_001(modes1,modes2)) + pi,barSize);
zlim([0 2*pi])
set( gca, 'ztick', [-pi 0 pi], 'zticklabel', {'-\pi', '0', '\pi'}, 'TickLabelInterpreter', 'tex')
view(2)
originalSize1 = get(gca, 'Position');
c = colorbar;
c.Limits = [0 2*pi];
c.Ticks  = [0 pi 2*pi];
c.TickLabels = {'-\pi', '0', '\pi'};
c.Label.String = 'Phase';
c.Label.Position(1) = savePos;

set(gca, 'Position', originalSize1); 
colormap(gca,twilight)

subLabels = {'a)', 'b)', 'c)', 'd)'};

for ii = 1 :4
    axes(ha(ii))
    for jj = 1:length(h{ii})
        zdata = get(h{ii}(jj),'Zdata');
        set(h{ii}(jj),'Cdata',zdata)
    end
    text( -.25, 1.1, .9, subLabels{ii}, 'units', 'normalized' )
    set( gca, 'xtick', linspace(1,length(modes2),2), 'ytick', linspace(1,length(modes1),2));
    xlim([.25 12.75])
    ylim([.25 12.75])
    FancyPlot(1)
    
    if ii > 2
        xL = xlabel('Mode in');
        xL.Position(2) = xL.Position(2)-2;
    end
    if ii == 1 || ii == 3
        yL = ylabel('Mode out');
        yL.Position(1) = yL.Position(1)+2;
    end
    
    % if no view(2):
%     text( 0, 1, .9, subLabels{ii}, 'units', 'normalized' )
%     set(get(gca,'ylabel'),'rotation',-38)
%     set(get(gca,'xlabel'),'rotation', 22)

end


set(findall(gcf,'-property','FontSize'),'FontSize',10)

% set(findall(gcf,'-property','FontName'),'FontName','Helvetica')


%% Check pump power drift and total counts per input, per matrix

clc
close all
plotPowerDrift( Power, Rates, RatesC2, inputStrS)





%% Function space - so you don't have to deal with ugly code ;)
% single rates (experimental)
function [Rates, Power, SingleRef, Rates2] = getRates( Folder, matrices, inputStrS, matStr)
% get the rates from the single measurements, without any correction on
% detection efficiencies. 
% Input
%   Folder      The Folder where the data is stored
%   matrices    List of all matrices to load
%   inputStrS   cell of strings of all input configurations for singles
%   matStr      String of matrixnames (e.g., 'Matrix_001' or 'Matrix_123')
% 
% Output
%   Rates       The rates (uncorrected for losses), split out for all
%               matrices in an object. E.g.,: 'Rates.Matrix_001'    
%   Power       Object with the pump power for each measurement, per matrix
% 
% optional:
%   SingleRef   Object with all reference counts (input 1) per matrix
%   Rates2      Only the singles, no heralding thingy

nOutput = 12;
nInput  = length( inputStrS);

% loop through the matrices
relCnts = zeros(max(matrices), nOutput);
for ii = matrices
    
    % loop through the different input configurations
    cnts1 = zeros( nOutput);
    cnts2 = zeros( nOutput);
    cnts3 = zeros( nOutput);
    cnts4 = zeros( nOutput);
    Power.(matStr{ii}) = zeros(1,nInput);
    
    for jj = 1 : nInput

        % select the data to load in
        Folder0 = [Folder];
        naam1 = ['202112*_data_*_inputConfig_',inputStrS{jj}, '_singles2.mat'];
        naam2 = ['202112*_data_*_inputConfig_',inputStrS{jj}, '_singles1.mat'];

        % load in the data
        files1 = dir([Folder0,naam1]);
        files2 = dir([Folder0,naam2]);
        data1 = load( [Folder0, files1.name]);
        data2 = load( [Folder0, files2.name]);
        
        
        % get all coincidence counts
        % get names of the swabian channels 
        sNames = fieldnames( data1.data.Coincidences);
        nNames = length( sNames);
        
        % extract all counts
        singleCnt1 = zeros( 1, nNames);
        singleCnt2 = zeros( 1, nNames);
        output = zeros(1, nNames);
        for ss = 1 : nNames
            singleCnt1(ss) = sum( data1.data.Coincidences.(sNames{ss}));
            singleCnt2(ss) = sum( data2.data.Coincidences.(sNames{ss}));
            output(ss) = getOutput( sNames{ss},0);
        end
        
        % order it by chip channel, not swabian mode
        if sum( output==0)>1
            warning('Some unknown outputmodes present')
        end
        [~,ind] = sort( output);
        cnts1(jj,:) = singleCnt1(ind);
        cnts2(jj,:) = singleCnt2(ind);
        Power.(matStr{ii})(jj) = data1.data.Pump.Power(2);
        
        
        % now extract all single counts
        sNames = fieldnames( data1.data.Singles);
        nNames = length(sNames);
        singleCnt3 = zeros( 1, nNames);
        singleCnt4 = zeros( 1, nNames);
        
        output = []; ind = [];
        for ss = 1 : nNames-1 %skip C15: the trigger
            singleCnt3(ss) = sum( data1.data.Singles.(sNames{ss}));
            singleCnt4(ss) = sum( data2.data.Singles.(sNames{ss}));
            output(ss) = getOutput( sNames{ss},0);
        end
        
        % order it by chip channel, not swabian mode
        if sum( output==0)>1
            warning('Some unknown outputmodes present')
        end
        [~,ind] = sort( output);
        cnts3(jj,:) = singleCnt3(ind);
        cnts4(jj,:) = singleCnt4(ind);
    end
    
    % now restructure the data such that it follows the matrix
    Rates.(matStr{ii}) = zeros(12);
    Rates2.(matStr{ii}) = zeros(12);

    Rates.(matStr{ii})(:,1) = cnts1(1,:)';    
    Rates2.(matStr{ii})(:,1) = cnts3(1,:)';
    for jj = 1:11
        Rates.(matStr{ii}) (:,jj+1) = cnts2(jj,:)';
        Rates2.(matStr{ii})(:,jj+1) = cnts4(jj,:)';
    end
    
    SingleRef.(matStr{ii}) = cnts1(1:nInput,:);
end



end

function RatesC = correctRates( matrices, matStr, Rates, Power, relEff, inputLoss, SingleRef, normCol)
% Correct the raw rates for the relative detection efficiencies and pump
% power (assume linear dependence for the latter, as differences are small
% anyway)
% Optional functionality: normalize each column
% 
% Input
%   matrices    Array of matrices you want to process
%   matStr      A cell with the matrix names ('Matrix_124')
%   Rates       The Rates data, in an object
%   Power       Object with all pump powers, per marix per input config
%   relEff      The relative efficiencies, in an object
%   inputLoss   An 1x12 array of specified (by Quix) incoupling losses
%   SingleRef   An object with all relative counts (input 1) per matrix
% Optional:
%   normcol     if 1, normalize each column (default: 1)
% 
% Output
%   RatesC      The corrected rates

% input handling
if nargin == 7
    % you always want to normalize each dataset as the pump power /
    % incoupling efficiency can vary over time
    normCol = 1;
end

% some settings
nModes = 12;

% get the relative detection efficiency inside a matrix
outputLoss = ones( nModes);
outputLoss(:,1) = relEff{1}';

for ii = 2 : 11
    outputLoss(:,ii) = relEff{ii-1}';   
end


% go through all matrices and correct for the efficiency and relative pump
% power
for ii = matrices
    
    % get relative pump power
    p = Power.(matStr{ii});
%     normPump = [p(1), p]/max(p);
    normPump = ones(1,12);
    
    % correct for the detection efficiencies and pump power
    RatesC.(matStr{ii}) = Rates.(matStr{ii}) ./outputLoss ./inputLoss' ./normPump;
    

    % normalize each column
    if normCol == 1
        for jj = 1 : nModes
            A = sum( RatesC.(matStr{ii})(:,jj));
            if A == 0
                A = 1;
            end
            RatesC.(matStr{ii})(:,jj) = RatesC.(matStr{ii})(:,jj) / A;
        end
        
% %         % now try to normalize each row (does not do much: /skip)
%         etaRange = 0.5 : 0.005 : 2;
%         nEta = length(etaRange);
%         etaOut = ones(1,12);
%         etaIn  = ones(1,12);
%         for nn = 1 : 8
%             
%             % optimize rows, loop throuhg all of them
%             for jj = 1 : 12
%                 
%                 % loop through all possible efficiencies
%                 distRow = zeros(1,nEta);
%                 for rr = 1 : nEta
%                     etaOut(jj) = etaRange(rr);
%                     
%                     B = RatesC.(matStr{ii}) ./ etaOut';
%                     distRow(rr) = std(1-sum(B'));
%                 end
%                 % take the best value (if multiple: select random one)
%                 ind = find( distRow == min(distRow));
%                 etaOut(jj) = etaRange( ind( randi(length(ind),1,1)));
%             end
%             
%             % optimize columns, loop through all of them
%             for jj = 1 : 12
%                 
%                 % loop through all possible efficiencies
%                 distRow = zeros(1,nEta);
%                 for rr = 1 : nEta
%                     etaIn(jj) = etaRange(rr);
%                     
%                     B = RatesC.(matStr{ii}) ./ etaIn;
%                     distRow(rr) = std(1-sum(B));
%                 end
%                 ind = find( distRow == min(distRow));
%                 etaIn(jj) = etaRange( ind( randi(length(ind),1,1)));
%             end
%         end
%         RatesC.(matStr{ii}) = RatesC.(matStr{ii}) ./ etaOut' ./ etaIn;
    end
    
    

end
end



% HOM visibilities (experimental)
function [dataT, Visibility, HOMposition] = processCoincidences(...
    Folder, nameC, matrices, inputStr, matStr, autoAccept, doPlotC, doPlotFit, nMoving, nFit )
% extract the coincidences counts
% Input
%   Folder          The folder where to find the data (excl. matrix)
%   nameC           The prefix filename (incl. *), no inputConfig_01_02_HOM
%   matrices        An array with the matrices you want to process
%   inputStr        A cell with strings of the input configurations
%   matStr          A cell with all matrix name strings
%   autoAccept      If 1, never ask for input to accept / reject a fit
%   doPlotC         If 1, plot all HOMs for 1 dip
%   doPlotFit       If 1, plot each individual fit
%   nMoving         # datapoints used for moving averaging (>=  1)
%   nFit            # datapoints used for the fit (around min/max) (>=1)
% 
% Output
%   dataT           Object with the raw data as it was saved / loaded
%                   Object per matrix, each matrix has a cell with data
%                   per input configuration
%   Visibility      Object with the visibilities per matrix
%   HOMposition     Object with the position of each HOM dip per matrix

% loop through all matrices
for ii =  matrices
    
    % select the correct folder
    Folder0 = [Folder];
    
    % loop through all input configurations
    for jj = 1 : length( inputStr)
        
        % build up the filename
        naam = [nameC, 'inputConfig_', inputStr{jj},'_HOM.mat'];
        % identify the file and load it in
        filesC  = dir([Folder0, naam]);
        nFilesC = length(filesC);
        
        % sanity check
        if nFilesC > 1
            warning( 'Too many files found')
        elseif nFilesC < 1
            warning( 'No files found')
        end
        
        % load in the data and store it in dataT
        temp = load( [Folder0, filesC(1).name]);
        data = temp.data;
        dataT.(matStr{ii}){jj} = data;
        
        % get all coincidence channel names
        cNames = fieldnames( data.Coincidences);
        nNames = length(cNames);
        
        % get the # datapoints outside the dip
        nOffset = getOffset( data.PositionList);
        
        % setup the right order of HOM dips
        nModes  = 12;
        outputs = nchoosek(1:nModes,2);
        swabianCh{length(outputs)} = [];
        for cc = 1 : length(outputs)
            swabianCh{cc}= getOutput( outputs(cc,:), 1);
        end
        if length(outputs) ~= nNames
           warning('Number of channels do not match') 
        end
        
        % loop through all coin. channels and get the corresponding HOM dip
        V = zeros(1,nNames);
        x = zeros(1,nNames);
        for cc = 1 : nNames
            [V(1,cc),x(cc)] = getVisibilityExp( data.PositionList,...
                data.Coincidences.(swabianCh{cc}), nOffset, nFit, autoAccept, doPlotFit, nMoving, cNames{cc});
        end
        
        % store the HOM position and Visibility
        HOMposition.(matStr{ii})(jj,:) = x;
        Visibility.(matStr{ii}) (jj,:) = V;
        if doPlotC == 1
            figure()
            clf
            hold on
            for cc = 1 : nNames
                plot( data.PositionList, ...
                    mean(data.Coincidences.(cNames{cc}))/...
                    mean(mean(data.Coincidences.(cNames{cc})(:,1:nOffset))),'o-')
            end
            legend( cNames)
            xlabel('Position (mm)')
            ylabel('Coincidence counts (Hz)')
            title( filesC.name(30:end-8), 'interpreter', 'none')
            ylim([0 2])
            
            box on
        end
    end
clc
end
end

function [V,x,C,Q] = getVisibilityExp( xData, yData, nOffset, nFit, autoAccept, plotFit, nMove, cName)
% gets the HOM visibility for the set xData and yData
% Input parameters
% 	xData       array with positions 	(1,N) in mm
% 	yData       array with coincidences (n,N) in Hz
% 	nOffset     # datapoints outside the HOM dip (assumes first datapoints)
%   nFit        # datapoints around min/max of HOM to fit
%   autoAccept  If 1, auto accept any fit of the HOM dip/peak
%   plotFit     1 if you want a plot of the Fit
%   nMove       # datapoints used for moving average
% optinal
%   cName       string with the measurement name
% 
% Output:
% 	V 		Visibility ( |V| <= 1)
%   x       The position of the dip, in mm
%   optional:
%   C       Classical: average counts outside HOM dip/peak
%   Q       Quantum: average counts inside dip / peak


% input validation
if nMove < 2
    nMove = 1;
end
if mod(nFit,2) == 1
    nFit = nFit+1;
end
if nFit < 4
    nFit = 4;
end


% get the average counts (and remove 0's because of rebiassing)
meanY = getAverageCnt( yData);


% get distinguishable photons
C = mean( meanY(1:nOffset));


% get the HOM dip/peak by fitting
if nargin == 8
    [~,~,Q,x] = fitHOM( xData, meanY, C, plotFit, nMove, nOffset, nFit, autoAccept, cName);
else
    [~,~,Q,x] = fitHOM( xData, meanY, C, plotFit, nMove, nOffset, nFit, autoAccept);
end

% get the visibility
 V = 1 - Q/C;
 
 % sanity checks
 if V < -1
	V = -1;
elseif V > 1
	V = 1;
end
end

function [xFit, yFit, HOM, x] = fitHOM( xData, yData, C, plotFit, nMove, nOffset, nFit, autoAccept, cName)
% fits the HOM with a local polynomial around the dip/peak, returns array with fit data
% input
%   xData     	array with positions, (1,N) in mm!
%   yData     	array with coincidences, (1,N)
%	C 			average coincidences for dist. photons 
%   plotFit     1 if you want a plot of the fit
%   nMove       # datapoints used for moving average
%   nFit        # datapoints around min/max of the HOM for the fit
%   autoAccept  If 1, automatically accept any hom dip fit
% optional:
%   cName       string with the measurement channel name
% 
% output
%	xFit 		array with positions of the fit (mm)
%	yFit 		array with fitted coincidences 
% 	HOM 		minimum of maximum of HOM (number)
%   x           the position of the dip (mm)

% in case the optional input is not provided
if nargin == 8
    cName = '';
end

% average over all the datapoints at 1 position
if size(yData,1) > 1
    Y = mean( yData(nOffset+1: end));
else
    Y = yData(nOffset+1 : end);
end
X = xData( nOffset+1 : end);

% apply moving average
Yavg = movmean( Y, nMove);

% find out if we have a dip or peak
% and set the initial conditions accordingly
testPeak = abs( max(Yavg) - C);
testDip  = abs( min(Yavg) - C);

% determine if peak/dip and find the index of the center datapoint in HOM
if testPeak > testDip
	% peak
	Amplitude = 1;
	ind = find( max(Yavg) == Yavg);
else
	% a dip
	Amplitude = -1;
    ind = find( min(Yavg) == Yavg);
end

% get the indices of the values around it
% and protect against negative || too large indices
nAround = nFit;
indFit = ind-nAround/2 : ind+nAround/2;
indWrong = find(indFit<1);
if ~isempty( indWrong)
%     warning('HOM dip/peak found at left edge!')
    indFit(indWrong) = [];
    indWrong = [];
end
indWrong = find( indFit>length(Yavg));
if ~isempty( indWrong)
%     warning('HOM dip/peak found at right edge!')
    indFit(indWrong) = [];
end


if isempty(indFit)
    warning('No indices are usable')
    
else
    % fit a polynomial through the minimum
    [ft,gof] = fit( X(indFit)', Yavg(indFit)', 'poly2' );
    xFit = min(X(indFit)) : 0.0005 : max(X(indFit));
    yFit = ft.p1*xFit.^2 + ft.p2*xFit.^1 + ft.p3;
    
    threshold = .75;
   if gof.rsquare < threshold && autoAccept ~= 1
       plotFit = 1;
       deleteFit = 1;
   else 
       deleteFit = 0;
   end
   
    % if wanted: plot the fit
    if plotFit == 1
        figure; clf
        hold on
        plot( xFit, yFit)
        plot( [xData(1:nOffset), X], [yData(1:nOffset), Y], 'o')
        hold off
        
        tekst{1} = ['sse = ', num2str(gof.sse)];
        tekst{2} = ['r2  = ', num2str(gof.rsquare)];
        tekst{3} = ['dfe = ', num2str(gof.dfe)];
        tekst{4} = ['R2a = ', num2str(gof.adjrsquare)];
        tekst{5} = ['rme = ', num2str(gof.rmse)];
        text( .2, .8, tekst, 'units', 'normalized')
        
        if gof.rsquare < threshold
            title(['Warning: ', cName, ' is terrible! '])
        end
    end
    
    if gof.rsquare < threshold && autoAccept ~= 1
        prompt = 'Type 1 if you want to use this fit ';
        doFit = input(prompt);
        if deleteFit == 1
            close()
        end
    else
        doFit = 1;
    end
    
    if doFit == 1
        % extract the HOM dip or peak
        if Amplitude == 1
            [HOM,x] = max( yFit);
            x = xFit(x);
        else
            [HOM,x] = min( yFit);
            x = xFit(x);
        end
    else
        HOM = C;
        x = 0;
    end
end
end

function V = getRefVisibility( Folder, Filename)
% Returns the visibility of the reference measurement (with an 50:50 BS on
% chip). This is used to correct for the limited distinguishability
% Input
%   Folder      The folder, string
%   Filename    The *exact* filename, string
% Ouput
%   V   the visibility (%)


% import the data
if ~strcmp( Filename(end-3:end), '.mat')
    Filename = [Filename, '.mat'];
end
files = dir( [Folder, Filename]);

if length( files) > 2
    warning('too many files found')
end
load( [Folder, files(1).name])


% find the channel with the coincidences, this should have the max. #counts
cNames = fieldnames( data.Coincidences);
nNames = length( cNames);

cntHOM = zeros( 1, nNames);
for ii = 1 : nNames
    cntHOM(ii) = mean(data.Coincidences.(cNames{ii})(1,:));
end
% select the one with the most counts
ind = find( cntHOM == max(cntHOM));

% get the visibility
% don't do moving averaging and don't plot (Will be fine anyway)
% automatically retrieve # offset datapoints ( assumes that the same
% position is used for all offset / out of dip measurements )
nOffset = getOffset( data.PositionList);
[V,~,~,~] = getVisibilityExp( data.PositionList, data.Coincidences.(cNames{ind}), nOffset, 4, 1, 1, 0);

end

function nOffset = getOffset( xData)
% automatically find the number of datapoints outside the HOM dip (assumes
% that these are measured at EXACTLY the same position, on the 'left' of
% the dip, i.e., stage position outside dip < stage position in dip
% Input
%   xData       Array of positions of the stage at which is measured
% Output
%   nOffset     # datapoints outside the HOM dip (default = 1)

ind = find( xData == xData(1));
nOffset = length(ind);
% output validation
if nOffset < 1
    nOffset = 1;
end
end

function orderedHOM = fixOrderHOMs( HOMe, matrices, inputStr, matStr)
% Returns a subset of the HOM dips, only the ones which are needed for the
% Obrien characterization. Furthermore, the HOM dips are ordered in the
% right order for the Obrien method
% 
% Input
%   HOMe        An object of experimental HOM dips, e.g., HOMe.Matrix_001
%   matrices    An array which lists all matrices to process
%   inputStr    A cell array with strings of all input configurations
%   matStr      A cell array with strings of all matrix names
% 
% Ouput
%   orderedHOM  An object with the correctly ordered HOM dips, per matrix

% how the experimental HOM dips are ordered
outputExp = nchoosek(1:12,2);

% loop through all matricse
for nn =  matrices
    
    % allocate memory for the new visibilities
    orderedHOM.(matStr{nn}) = zeros( 12, 12, 2, 2);
    for gg = 2 : 12
        
        for hh = 2 : 12
            
            for jj = 1 : 2
                
                for kk = 1 : 2
                                        
                    % find corresponding output
                    output = [jj, gg];
                    indOut = find( ismember(outputExp, output,'rows') ==1);
                    
                    % find corresponding input
                    inputS = ['0', num2str(kk), '_', num2str(hh,'%02.f')];
                    indIn  = find( contains( inputStr, inputS));
                    
                    % get the visibility & store the result
                    temp = HOMe.(matStr{nn})(indIn,indOut);
                    if isempty(temp) || isnan(temp)
                        orderedHOM.(matStr{nn})(gg,hh,jj,kk) = 0;
                    else
                        orderedHOM.(matStr{nn})(gg,hh,jj,kk) = temp;
                    end
                end
            end
        end
    end
end
end



% target matrix functions
function Ut = processTargetMatrix()
% import the Quix target matrix
% returns Ut.Matrix_001

filename = 'M:\Data\QuantumLoop_Gravity\Attempt 1\The matrices\matrix_Peter_PeterBilliardsAndGravity_matrix_4.txt';
dataLines = [1, Inf];

%% Setup the Import Options and import the data
opts = delimitedTextImportOptions("NumVariables", 13);

% Specify range and delimiter
opts.DataLines = dataLines;
opts.Delimiter = "(";

% Specify column names and types
opts.VariableNames = ["Var1", "e030000000000000000000e00j", "e218009190066423690628e37j", "e200000000000000000000e00j", "e024526720455268780172e18j", "e011171773033762790002e16j", "e054482207284879310232e21j", "e057262155518943799300e21j", "e010000000000000000000e00j", "e088511295254074099822e24j", "e020000000000000000000e00j", "e020000000000000000000e00j_1", "e041416635564493970095e20j"];
opts.SelectedVariableNames = ["e030000000000000000000e00j", "e218009190066423690628e37j", "e200000000000000000000e00j", "e024526720455268780172e18j", "e011171773033762790002e16j", "e054482207284879310232e21j", "e057262155518943799300e21j", "e010000000000000000000e00j", "e088511295254074099822e24j", "e020000000000000000000e00j", "e020000000000000000000e00j_1", "e041416635564493970095e20j"];
opts.VariableTypes = ["char", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double", "double"];

% Specify file level properties
opts.ExtraColumnsRule = "ignore";
opts.EmptyLineRule = "read";

% Specify variable properties
opts = setvaropts(opts, "Var1", "WhitespaceRule", "preserve");
opts = setvaropts(opts, "Var1", "EmptyFieldRule", "auto");
opts = setvaropts(opts, ["e030000000000000000000e00j", "e218009190066423690628e37j", "e200000000000000000000e00j", "e024526720455268780172e18j", "e011171773033762790002e16j", "e054482207284879310232e21j", "e057262155518943799300e21j", "e010000000000000000000e00j", "e088511295254074099822e24j", "e020000000000000000000e00j", "e020000000000000000000e00j_1", "e041416635564493970095e20j"], "TrimNonNumeric", true);
opts = setvaropts(opts, ["e030000000000000000000e00j", "e218009190066423690628e37j", "e200000000000000000000e00j", "e024526720455268780172e18j", "e011171773033762790002e16j", "e054482207284879310232e21j", "e057262155518943799300e21j", "e010000000000000000000e00j", "e088511295254074099822e24j", "e020000000000000000000e00j", "e020000000000000000000e00j_1", "e041416635564493970095e20j"], "ThousandsSeparator", ",");

% Import the data
A = readtable(filename, opts);

%% Convert to output type
Ut.Matrix_001 = table2array(A);
end

function HOM = getHOMTarget( U, inputModes)
% returns the HOM visibilities of the ideal matrices
% Input
%   U           An object of 1 or more nxn matrices (e.g., U.Matrix1)
%   inputModes  Array of the used input modes (e.g., [1 2])
% Output
%   HOM         An object with the visibilities per matrix (HOM.Matrix1)

nModes  = 12;
outputs = nchoosek(1:nModes,2);
nOutputs= length(outputs);
nInputs = length(inputModes);
uNames  = fieldnames(U);
nNames  = length(uNames);

% loop through all matrices in object U
for ii = 1 : nNames
    % go trough all possible 2-photon HOM dips
    for jj = 1 : nOutputs
        % go through all input modes
        for kk = 1 : nInputs
            
            % get the submatrix and compute the HOM dip
            Usub = U.(uNames{ii})(outputs(jj,:),inputModes{kk});
            C = permanentRyser( abs( Usub).^2);
            Q = abs( permanentRyser( Usub)).^2;
            % in case matrix element = 0: set V = 0 
            if C > 0
                HOM.(uNames{ii})(kk,jj) = 1 - Q/C;
            else
                HOM.(uNames{ii})(kk,jj) = 0;
            end
        end
    end
end
end



% (converting between) channel names
function output = getOutput( channels, type)
% Converts the list of Swabian channels to the outputmodes of the chip
% works for singles only (type = 0), or converts chip channels into swabian
% Channels (type=1)
% Input
%   channels    list of channels
%               type 0: string ('C9'), it removes C15 triggerdiode
%               type 1: array ([1 3])
% optional:
%   type        0 or 1  (default: 0)
% 
% Output
%   output      type 0: integer, chip mode
%               type 1: string, Swabian channels ('C9C10')

% hardcoded list
% Chip  Detector    Swabian
%  1    A1           9
%  2    A2          10
%  3    A3          11
%  4    A4          12
%  5    B1           6
%  6    B2           7
%  7    B3           8
%  8    C1          14
%  9    C2           5
% 10    C3           2
% 11    C4           4
% 12    D2           1

if nargin == 1
    type = 0;
end

% Convert Swabian Channel to chip mode
if type == 0
    % remove the C15 trigger
    if length( channels) > 3
        if strcmp( channels(end-2:end), 'C15')
            channels = channels(1:end-3);
        end
    end
    switch channels
        case 'C9'
            output = 1;
        case 'C10'
            output = 2;
        case 'C11'
            output = 3;
        case 'C12'
            output = 4;
        case 'C6'
            output = 5;
        case 'C7'
            output = 6;
        case 'C8'
            output = 7;
        case 'C14'
            output = 8;
        case 'C5'
            output = 9;
        case 'C2'
            output = 10;
        case 'C4'
            output = 11;
        case 'C1'
            output = 12;
        otherwise
            output = 0;
    end
% convert chip mode to Swabian Channel
elseif type == 1 
    temp = [];
    for ii = 1 : length( channels)
        switch channels(ii)
            case 1
                str = 9;
            case 2
                str = 10;
            case 3
                str = 11;
            case 4
                str = 12;
            case 5
                str = 6;
            case 6
                str = 7;
            case 7
                str = 8;
            case 8
                str = 14;
            case 9
                str = 5;
            case 10
                str = 2;
            case 11
                str = 4;
            case 12
                str = 1;
            otherwise
                str = 0;
                warning('Incorrect chip mode number specified')
        end
        temp = [temp, str];
    end
    % now sort in ascending order
    temp = sort(temp);
    output = '';
    for ii = 1 : length(channels)
        output = [output, 'C',num2str(temp(ii))];
    end
end
    
end
                                            
function [inputModes, inputStr, inputStrS] = getInputModes( nMax)
% get a cell with all input modes as int and 1 as string. All input
% modes are listed as: 1-2, 1-3, ... 1-12 and 2-3, 2-4,... 2-12
%
% Optional input;
%   nMax            Number, the highest # inputmodes you want to do for
%                   the matrix verification (default: 12)
%                   Example, for a 6x6 submatix, nMax = 6;
% 
% Output:
%   inputModes      an cell of 9, of all input modes (integers)
%   inputStr        an cell of 9, with strings of the input modes
%   inputStrS       an cell of 21, with strings for all inputs for singles

% input validation
if nargin == 0
    nMax = 12;
elseif nMax < 1
    nMax = 1;
elseif nMax > 12
    nMax = 12;
end


ind = 1;
inputModes{nMax*2-3} = zeros(1,2);
inputStr{nMax*2-3} = '';
inputStrS{nMax-1} = '';
for ii = 2 : nMax
    inputModes{ind} = [1 ii];
    inputStr{ind} = ['01_', num2str(ii,'%02.f')];
    inputStrS{ind}= inputStr{ind};
    ind = ind+1;
end
ind0 = ind;
for ii = nMax+1 : 12
    inputStrS{ind0} = ['01_', num2str(ii,'%02.f')];
    ind0=ind0+1;
end
for ii = 3 : nMax
    inputModes{ind} = [2 ii];
    inputStr{ind} = ['02_', num2str(ii,'%02.f')];
    ind = ind+1;
end
end

function matStr = getMatrixString( nMatrices)
% get string handle for all matrices
% input
%   nMatrices   total # of matrices
% output
%   matStr      a cell with all strings ('Matrix_001', or 'Matrix_123')

matStr{nMatrices} = 'Matrix_000';
for ii = 1 : nMatrices
    matStr{ii} = ['Matrix_', num2str(ii,'%03.f')];
end
end



% data validation (internal)
function meanCounts = getAverageCnt( counts)
% Returns the average number of counts for each timestep, but it omits the
% data = 0 as it is (probably) the result of rebiasing
% 
% Input
%   counts      nxm marix with the counts. The entries of each column are
%               repeated measurements at the same position / settings
% 
% Output
%   meanCounts  array with the average counts per timestep, where the
%               cases where the detectors rebiassed are removed

% to do: check previous entry before the 0 as well (might be lower than
% expected and thus should be omitted)
% alternative idea: check if the standard deviation is too large or not
[row,col] = find( counts == 0);
meanCounts = zeros(1,size( counts,2));
for ii = 1 : length( meanCounts)
    temp = counts(:,ii);
    if ismember( ii, col)
        rows = find( counts(:,ii) == 0);
        temp(rows) = [];
    end
    
    % in case all values are 0, then it probably was 0 so return 0 in that
    % case (and prevent errors of averaging over an empty set)
    if isempty( temp)
        meanCounts(ii) = 0;
    else
        meanCounts(ii) = mean( temp);
    end

end
end

function [Distance, x2] = getEffectivePurity( HOMt, HOMe, doPlot)
% tries a range of effective indistinguishabilities (HOM dips) and for each
% one, correct the experimental HOM dips and compute the trace distance of 
% the theoretically expected HOM dips
%
% Input
%   HOMt        Array of expected HOM dips
%   HOMe        Array of experimental observed HOM dips
%   doPlot      if 1, plot fidelity vs purity
% 
% Ouput
%   Distance    The best observed trace distance
%   x2          The corresponding best matching HOM dip


% the range you want to scan
x = 0.01 : 0.00025 :  1;
nSteps = length(x);

% try all purities
D = zeros(1,nSteps);
Dt= zeros(1,nSteps);
C = zeros(1,nSteps);
for ii = 1 : nSteps
    D(ii) = 0.5*sum( abs( HOMt- HOMe/x(ii)));
    Dt(ii)=     max( abs( HOMt- HOMe/x(ii)));
    C(ii) = 0.5*sum( abs( (HOMt-HOMe/x(ii))./HOMt));
end


% get the best ones
ind1 = find( D == min(D));
ind2 = find( Dt== min(Dt));
ind3 = find( C == min(C));
x2 = x( ind1(1));
xb = x( ind2(1));
xc = x( ind3(1));
Distance = D( ind1(1));


if doPlot == 1
    figure
    clf
    subplot(3,1,1)
    plot( x, D, '.-')
    ylabel('Distance')
    ylim([0 max(ylim)])
    xlim([0 1])
    set( gca, 'xtick', sort( unique([0 x2 1])),...
        'yscale', 'log')
    FancyPlot( get( gcf, 'Number'), 3)
    
    subplot(3,1,2)
    plot( x, Dt, '.-')
    ylabel('Max distance')
    ylim([0 max(ylim)])
    xlim([0 1])
    set( gca, 'xtick', sort( unique([0 xb 1])),...
        'yscale', 'log')
    FancyPlot( get( gcf, 'Number'), 3)
    
    subplot(3,1,3)
    plot( x, C, '.-')
    xlabel('Effective HOM visibility')
    ylabel('Trace distance (relative)')
    xlim([0 1])
    ylim([0 max(ylim)])
    set( gca, 'xtick', sort( unique([0 xc 1])),...
        'yscale', 'log')
    FancyPlot( get( gcf, 'Number'), 3)
end



end



% Process the data, get the eigenvalue distributions
function [tList, U, F, dataR] = getEigenvaluesSpectra( matrices, Ut,...
    RatesC, RatesC2, HOMe, matStr, RorT, doPlotFidelity)
% Process the data with the Obrien characterization and return the singular
% transmission eigenvalues
% 
% Input
%   matrices        Array of all matrices to process
%   Ut              Object of all ideal matrices
%   RatesC          Object of all (normalized) single counts
%   RatesC2         Object of all unnormalized single counts
%   HOMe            Object with all exp. HOM dips (ordered for Obrien)
%   matStr          Cell array with all matrix names (string)
%   RorT            If 'R', get the reflection matrix
%                   If 'T', or other, go for the transmission matrix
% optional:
%   doPlotFidelity  If 1, plot the fidelity (default: 0)
% 
% Output
%   tList           Object with the singular values, split out per type,
%                   per matrix (e.g.: tList.Amp.Matrix_001)
%                   .Amp        Only amplitudes (From rates)
%                   .Phase      Rates + phases (from BEFORE polar)
%                   .PhasePolar Rates + phases (from AFTER polar)
%                   .Polar      Reconstructed matrix, after polar
%                   .Ideal      The ideal matrix
% 
% optional:
%   U               Object with all types of reconstructed matrices
%                   .Amp        Only amplitudes (From rates)
%                   .Phase      Ampl. + phases (from BEFORE polar)
%                   .PhasePolar Ampl. + phases (from AFTER polar)
%                   .Polar      Reconstructed matrix, after polar
%                   .Ideal      The ideal matrix
%                   .unNorm     Unnormalized rates + phases (before polar) 
%   F               Array with matrix fidelities
%   dataR           Object with all the data (including the reconstructed
%                   matrix before the polar decomposition) per matrix

% input check
if nargin == 6
    doPlotFidelity = 0;
end

% selection of matrix elements for the transission matrix
if upper(RorT) == 'R'
    outputs = 1:6;
    inputs  = 1:6;
else
    outputs = 7:12;
    inputs  = 1: 6;
end


% loop through all the matrices
F = zeros( 1, max(matrices));
for ii = matrices
    % allocate memory
    if ii == 1
        tList.Amp{6}       = [];
        tList.Phase{6}     = [];
        tList.PhasePolar{6}= [];
        tList.Polar{6}     = [];
        tList.Ideal{6}     = [];
        tList.unNorm{6}    = [];
    end
    
    % do O'brien to get the phases correct
    rePhased = rephase(Ut.(matStr{ii}));
    [r, F(ii), dataR.(matStr{ii})] = simObrien_v2( rePhased,...
        sign(angle(rePhased(2,2))), RatesC.(matStr{ii}), HOMe.(matStr{ii}),0.9);
    
    
    % build up the array of matrices
    U.Amp.(matStr{ii})        = sqrt(RatesC.(matStr{ii}));
    U.Phase.(matStr{ii})      = sqrt(RatesC.(matStr{ii})) .* exp( 1i*angle( dataR.(matStr{ii}).Reconstructed));
    U.PhasePolar.(matStr{ii}) = sqrt(RatesC.(matStr{ii})) .* exp( 1i*angle( r));
    U.Polar.(matStr{ii})      = r;
    U.Ideal.(matStr{ii})      = Ut.(matStr{ii});
    U.unNorm.(matStr{ii})     = sqrt(RatesC2.(matStr{ii})) .* exp( 1i*angle( dataR.(matStr{ii}).Reconstructed));
    
    % get the names for all
    uNames = fieldnames(U);
    nNames = length(uNames);
    
    % select # modes of the transmission matrix you want to use
    for jj = 1 : 6
        % get the (partial) transmission matrices
        % randomly select the modes for the truncated transmission matrix
        modeOut= sort(randperm(6,7-jj))+outputs(1)-1;
        modeIn = sort(randperm(6,7-jj));

        % get for all types of matrices the singular values
        for uu = 1 : nNames
            T = U.(uNames{uu}).(matStr{ii})(modeOut,modeIn);
            
            % get the values and store them
            t = svd( T);
            tList.(uNames{uu}){jj} = [tList.(uNames{uu}){jj}, t.^2];
        end        
    end
end

% if wanted: plot the fidelities
if doPlotFidelity == 1
    figure
    clf
    bar( F')
    ylim([0 1])
    xlabel('Matrix')
    ylabel('Obrien Fidelity')
    FancyPlot( 3)
    xlim([-.5 max(matrices)+1.5])
end
end

function plotEigenvalueSpectra( Tlist, doLog, maxBin, doFit)
% plot the eigenvalue spectra for the given input modes
% Input
%   Tlist   Cell array with the singular values
%   doLog   If 1, make y-scale log, otherwise linear scale
% 
% optional
%   maxBin  Max bin edge (default: max of the data set)
%   doFit   If 1, fit a DMPK distribution through the 6/6 set
%           (default: 0)

if nargin == 2
    maxBin = max(max(Tlist{1}));
    doFit  = 0;
elseif nargin == 3
    doFit = 0;
end

% set up the bins of the histogram
if maxBin == 1
    nBins = 51;
else
    nBins = ceil(51 *maxBin);
end
bins  = linspace( 0, maxBin, nBins);

% open a figure window
figure
clf
[ha, ~] = tight_subplot(3,2, 0, [.15 0.1], [.1 .1]);

% loop through all transmission sizes
for jj = 1 : length(Tlist)
    
    Tlist2 = Tlist{jj}(:);
    
    axes(ha(jj))
    h = histogram(Tlist2, bins);
    % set lin/log scale
    if doLog == 1
        set( gca, 'yscale', 'log',...
            'ylim', [1 5*max(ylim)])
    else
        ylim([0 1.2*max(ylim)])
    end
    FancyPlot( get(gcf,'Number'), 3)
    
    % set y-axis to same scale for all subpanels
    if jj == 1
        maxY = max(ylim);
    else
        if doLog == 1
            ylim([1 maxY])
        else 
            ylim([0 maxY])
        end
    end

    % set yaxis labels / ticks
    if mod(jj,2) == 1
        ylabel( 'Occurence')
    else
        set( gca, 'yticklabel', '')
    end
    % set xaxis labels / ticks
    if jj >=5
        xlabel('Singular value \tau^2')
    else
        set( gca, 'xticklabel', '')
    end
    
    % indicate # modes used
    if doLog == 1
        yTxt = .5*maxY;
    else
        yTxt = .85*maxY;
    end
    text( 0.7*max(xlim), yTxt, [num2str(6+1-jj), '/6 modes'])
    
    % if suitable: return the average transmission
    if abs( maxBin - 1) < 1E-2 && jj == 1
        text( 0.1*max(xlim), yTxt, ['<T> = ', num2str( round( mean(Tlist2),3))])
    end
    
    if jj == 1 && doFit == 1
        [x,y] = fitDMPK( bins, Tlist2, [0.05 1]); 
        hold on
        plot( x,y, 'r')
        hold off
    end
        

end
end

function [x,y] = fitDMPK( xData, yData, fitRange)
% rescale the DMPK theory line such that it matched with the experimental
% data.
% 
% Input
%   xData       Array with bins of singular values
%   yData       Array with how often these bins are filled
% 
% optional:
%   fitRange    Array with 2 elements of increasing order, specifies what
%               range of singular values should be considered for the fit
%               default: [.1 .95]
% 
% Output
%   xFit        Array of singular values
%   yFit        Array of counts corresponding to the singular values


% get avgT
avgT = mean( yData(:));


% the region used for normalization
if nargin == 2
    xTest = [0.1, 0.95];
else
    xTest = fitRange;
end
yTest = histcounts( yData, xData);


% get the relevant datapoints (eigenvalues)
ind0 = find( xData > xTest(1));
ind  = find( xData(ind0) <xTest(2));
x = ind0(ind);
if isempty(x)
    warning('no data in the fitrange found')
end


% get the rescale factor
y = 1/2 /avgT ./xData(x) ./sqrt(1-xData(x));
rescaleFactor = mean(yTest(x) ./ y);


% now reconstruct the whole rescaled DMPK curve
xBegin = 0.01;
xStep  = 0.005;
xEnd   = 1-xBegin;
x = xBegin : xStep : xEnd;
y = rescaleFactor /2 /avgT ./x ./sqrt(1-x);
end



% visual data validation (nog nalopen!!)
function plotPhaseMatrices( R, Ut, matrices, matStr, inputStr)
% plots the phases of the reconstructed and target matrices
% returns a list of the experimental and theoretical expected phases of the 
% matrix elements
% 
% Input
%   R           The experimentally reconstructed matrix, in an object
%   Ut          The expected (theory) matrix, in an object
%   matrices    The matrix numbers you want to process (array)
%   matSt       Cell with the matrix names (string)
%   inputStr    Cell with the input configurations (string)


% loop through the matrices of interest
for ii = matrices
    
    % for each matrix: make a new figure
    figure
    clf
        
    A = angle(R.(matStr{ii})(:,jj));
    B = angle(Ut.(matStr{ii})(:,jj));
    bar( 1:12^2, [ A; B]);
    set( gca, 'xtick', 1:12:12^2,...
        'ytick', -pi:pi/2:pi,...
        'xlim', [0.5 12^2+.5],...
        'ylim', [-1 1]*pi*1.1,...
        'yticklabels', {'-\pi', '-\pi/2', '0', '\pi/2', '\pi'});
    
    ylabel('Matrix phases')
    xlabel('Matrix element')
    legend('Experimental','Expected');
    title( matStr{ii}, 'interpreter', 'none', 'fontweight', 'normal')
    
    FancyPlot( get(gcf,'number'),2)
end
end

function plotSingles( RatesC, Ut, matrices, matStr, inputStr)
% plots the singles of the experiment and target matrices
% returns a list of the experimental and theoretical amplitudes of the 
% matrix elements
% 
% Input
%   RatesC      The experimental rates, in an object
%   Ut          The expected (theory) matrix elements
%   matrices    The matrix numbers you want to process
%   matSt       Cell with the matrix names (string)
%   inputStr    Cell with the input configurations (string)


for ii = matrices
    figure
    clf
    [ha, ~] = tight_subplot(3,3, 0, [.15 0.1], [.1 .1]);
    for jj = 1 :4%: length(inputStr)
        
        axes(ha(jj));
        A = RatesC.(matStr{ii})(:,jj);
        B = abs(Ut.(matStr{ii})(:,jj)).^2;
        bar( 1:12, [ A/sum(A), B]);
        set( gca, 'xtick', 1:12,...
                  'ytick', 0:.2:.8,...
                  'xlim', [0.5 12.5],...
                  'ylim', [0 .75])
        if mod(jj,3) == 1
            set( gca, 'yticklabels', {'0', '0.2', '0.4', '0.6', ''});
            ylabel('Matrix amplitude')
        else
            set( gca, 'yticklabels','')
        end
        if jj>7
            set( gca, 'xticklabels', {'1', '', '3', '', '5', '', '7', '', '9', '', '11', ''});
            xlabel('Output mode')
        elseif jj == 3
            legend('Experimental','Expected',...
                'Position', [.87 .895 .0863 0.1127],...
                'box', 'off')
            set( gca, 'xticklabels','')
        elseif jj == 2
            title( matStr{ii}, 'interpreter', 'none', 'fontweight', 'normal')
            set( gca, 'xticklabels','')
        else
            set( gca, 'xticklabels','')
        end
        text(.05,.95,['Input ', inputStr{jj}], 'interpreter', 'none',...
            'units', 'normalized')
        text(.67,.95, ['C = ', num2str( round( corr(A,B),2))],...
            'units', 'normalized')
        FancyPlot( get(gcf,'number'),2)
    end
end
end

function plotHOMVisibilities( HOMe, HOMt, matrices, matStr, inputStr)
% Plots and compares the measured HOM visibilities with the expected, ideal
% HOM visibilities
% Input:    
%   HOMe        Object with the experimentaly determined HOM dips
%   HOMt        Object with the expected HOM dips
%   matrices    Array with the matrices to verify
%   matStr      Cell with the matrix names as string
%   inputStr    Cell with the input configurations as string
% 


% loop through all selected matrices
for ii = matrices
   
    figure
    clf
    [ha, ~] = tight_subplot(3,3, 0, [.15 0.1], [.1 .1]);
    
   
    %loop through all available input configutations
    for jj = 1 : 8%length(inputStr)

        A = HOMe.(matStr{ii})(jj,:);
        B = HOMt.(matStr{ii})(jj,:);
        
        axes(ha(jj))
        bar( 1:66, [A;B] );
        set( gca, 'xtick', 0:10:70,...
                  'ytick', -1:.5:1,...
                  'xlim', [0 67],...
                  'ylim', [-1 1]*1.2)

        % setup y axes
        if mod(jj,3) == 1
            set( gca, 'yticklabels', {'-1', '-0.5', '0', '0.5', '1'});
            ylabel('Visibilities')
        else
            set( gca, 'yticklabels','')
        end
        
        % setup x axes
        if jj > 7
            set( gca, 'xticklabels', {'0', '10', '20', '30', '40', '50', '60', '70'});
            xlabel('HOM nr')
        elseif jj == 2
            title( matStr{ii}, 'interpreter', 'none', 'fontweight', 'normal')
            set( gca, 'xticklabels', '')
        elseif jj == 3
            legend('Experimental','Expected',...
                'Position', [.87 .895 .0863 0.1127],...
                'box', 'off')
            set( gca, 'xticklabels', '')
        else
            set( gca, 'xticklabels','')
        end
        
        text(.05,.95,['Input ', inputStr{jj}], 'interpreter', 'none',...
            'units', 'normalized')
        text(.67,.95, ['C = ', num2str( round( corr(A',B'),2))],...
            'units', 'normalized')
        FancyPlot( get(gcf,'number'),2)

    end
end
end

function plotHOMposition( HOMposition, data, matrices, inputStr, matStr, plotEach, plotTotal)
% produces histograms of all the position where the HOM dip is determined
% input
%   HOMposition     the data to plot
%   data            Cell array of the saved data of matrix 001
%   matrices        An array with all matrices to consider
%   inputStr        A cell with all input configurations as string
%   matStr          A cell with all matrix names as string
%   plotEach        1 if you want a histogram of positions per matrix
%   plotTotal       1 if you want a histogram of positions of all matrices
% 


nBins = 30;
nMatrices = length(matrices);
total{21} = [];
% loop through all matrices
for ii = matrices
    
    nInputs = size( HOMposition.(matStr{ii}),1);

    if plotEach == 1
        figure
        clf
        yMax = 1;
        for jj = 1 : nInputs
            total{jj} = [total{jj}, HOMposition.(matStr{ii})(jj,:)];
            subplot(nInputs,1,jj)
            histogram( HOMposition.(matStr{ii})(jj,:),nBins);
            yMax = max( yMax, max(ylim));
        end
        
        subplot(nInputs,1,1)
        for jj = 1:nInputs
            subplot(nInputs,1,jj)
            ylabel('Occurences')
            ylim([ 0 yMax])
            title(['Input ', inputStr{jj}], 'interpreter', 'none')
        end
        xlabel('HOM dip/peak position (mm)')
        FancyPlot( get( gcf, 'Number'), 1)
    else
        for jj = 1 : nInputs
            total{jj} = [total{jj}, HOMposition.(matStr{ii})(jj,:)];
        end
    end

end
% plot all per input configuration
if plotTotal == 1
    figure
    clf
    for jj = 1 : nInputs
        subplot(ceil(nInputs/2),2,jj)
        histogram( total{jj}, nBins);
%         ylim([0 max(1e3, max(ylim))])
%         set( gca, 'xtick', (round(data{jj}.PositionList(2:2:end),2)),...
%             'xlim', [min( data{jj}.PositionList(1), min(xlim)), max( max(xlim), data{jj}.PositionList(end))])
%         ylabel('Occurences')
        text( .1, .95, ['Input config ', inputStr{jj}],...
            'units', 'normalized', 'interpreter', 'none')  
%         title(['Input config ', inputStr{jj}], 'interpreter', 'none')
        FancyPlot( get( gcf, 'Number'), 3)

    end
    xlabel('HOM dip/peak position (mm)')
    set( gcf, 'Name', 'HOM positions of all matrices')
end
end

function plotFidelities( Ut, R, dataR, matrices, matStr)
% plot all the fidelities of the matrix amplitudes only (top panel) and of
% the whole matrix (bottom panel)
% 
% Input
%   Ut          target matrices
%   R           object with normalized rates
%   dataR       object with phases in dataR.Matrix_001.Reconstructed
%   matrices    list of matrices to loop through
%   matStr      object of all matrix names (string)

Famp = zeros(1,max(matrices));
Fphs = zeros(1,max(matrices));
for ii = matrices
    A = sqrt(R.(matStr{ii})) .* exp( 1i*angle( dataR.(matStr{ii}).Reconstructed));
    Famp(ii) = getFidelity( abs( Ut.(matStr{ii})), abs(A));   
    Fphs(ii) = getFidelity(      Ut.(matStr{ii}),      A);   
end


figure
clf
[ha, ~] = tight_subplot(2,1, 0, [.15 0.1], [.1 .1]);

axes(ha(1))
hold on
bar( matrices, Famp)
plot( [min(matrices), max(matrices)], mean(Famp)*ones(1,2), '--')
text( max(xlim)*.65, 1, ['F = ', num2str(round( mean(Famp),2)), ' +/- ', num2str( round( std(Famp),2))])
hold off


axes(ha(2))
hold on
bar( matrices, Fphs)
plot( [min(matrices), max(matrices)], mean(Fphs)*ones(1,2), '--')
text( max(xlim)*.65, 1, ['F = ', num2str(round( mean(Fphs),2)), ' +/- ', num2str( round( std(Fphs),2))])
hold off

xlabel('DMPK matrix')

for ii = 1 :2
    axes(ha(ii))
    ylim([0 1.1])
    ylabel('Fidelity')
    xlim([min(matrices)-1 max(matrices)+1])
    set( gca, 'xtick', 0:50:200,...
        'ytick', 0 : .5 : 1,...
        'yticklabel', {'0', '0.5', '1'})
    FancyPlot(3)
end

set( gca, 'xticklabels', {'0', '50', '100', '150', '200'})

end

function getSingularTotal( U, matrices, matStr)
% retrives and plots the singular values of all reconstructed matrices
%
% Input
%   U           Object with all matrices (e.g., U.Ideal.Matrix_123)
%   matrices    List of all matrices to check
%   matStr      Cell array with all stings of all matrices
% 

% retrieve all types of reconstructed matrices
Names = fieldnames( U);
nNames= length( Names);

maxSVD = 0;
% loop through all types
for nn = 1 : nNames
    
    % loop through all requested matrices
    for ii = matrices
        % if first matrix: initialize
        if ii == 1
            tListTotal.(Names{nn}) = [];
        end
        % get the singular values
        temp = svd(U.(Names{nn}).(matStr{ii}));
        tListTotal.(Names{nn}) = [tListTotal.(Names{nn}); temp.^2];
    end
    
    % store the max value
    maxSVD = max( [maxSVD, max(max( tListTotal.(Names{nn})))]);
end

% plotting time
if nNames == 1
    nCols = 1;
    nRows = 1;
else
    nCols = 2;
    nRows = ceil( nNames/nCols);
end

figure
[ha, ~] = tight_subplot(nRows, nCols, 0, [.15 0.1], [.1 .1]);
for nn = 1 : max(nNames, nCols*nRows)
    axes( ha(nn))
    if mod(nn,2)
        yyaxis left
        set( gca, 'YColor', [0.1500 0.1500 0.1500],...
            'XColor', [0.1500 0.1500 0.1500])
    else
        yyaxis right
        set( gca, 'YColor', [0.1500 0.1500 0.1500],...
            'XColor', [0.1500 0.1500 0.1500])
    end
    
    if nn <= nNames
        histogram( tListTotal.(Names{nn}),linspace(0, maxSVD,100),...
            'FaceColor', [ 0    0.4470    0.7410])
    else
        xlim([0 maxSVD])
        ylim([0 1])
    end
    FancyPlot(3)
    if nn <= nNames
        text(0.4*maxSVD, 0.95*max(ylim), Names{nn})
    end
    if nn == 1
        xtick = get( gca, 'xtick');
        for ii = 1 : length(xtick)
            xticklbl{ii} = num2str( xtick(ii));
        end
    end

    set( gca, 'xtick', xtick, 'xticklabel', xticklbl,...
        'ylim', [0 max(ylim)*1.1],...
        'xlim', [-.05 maxSVD+.05])
    
    if nn > nNames - nCols
        xlabel('Singular values')
    end
end
end

function angles = getAngles( U, matrices, matStr, nCols)
% computes the angles between the columns for all matrices specified
% this serves as an indirect check of unitarity, mainly to check if the
% columnwise normalization of the rates is secretly imposing unitarity
% 
% input
%   U           Object with all matrices
%   matrices    Array of all matrices to check
%   matStr      Cell array with all matrix names
% optional
%   nCols       Max #cols to check (default: 12)
% 
% output
%   angles      Object with all angles, specified per type of matrix (as
%               given in U as input)

% input handling
if nargin == 3
    nCols = 12;
end

% get all types of matrices inside the object U
uNames = fieldnames(U);
nNames = length(uNames);

% loop through all types of U
for uu = 1 : nNames
    
    % loop through all matrices
    for ii = matrices
        
        angles.(uNames{uu}).(matStr{ii}) = zeros(nCols);
        % loop first vector through all columns
        for cc = 1 : nCols
            u = U.(uNames{uu}).(matStr{ii})(cc,:);

            % loop 2nd vector through all columns
            for rr = 1 : nCols
                v = U.(uNames{uu}).(matStr{ii})(rr,:);

                % compute the angle and store it
                a = acosd( abs( dot(u,v)) / norm(v) / norm(u));
                angles.(uNames{uu}).(matStr{ii})(cc,rr) = a;
            end
        end
    end
end

end

function plotAngles( angles, matrices, matStr)
% process and plot the angles
%
% Input
%   angles      Object with the angles
%   matrices    Array of all matrices
%   matStr      Cell array with all matrix names

% allocate memory
maxAngle = zeros(1,200);
storeAngle.Phase = zeros(36,200);
storeAngle.unNorm= zeros(36,200);

% loop through the matrices
for ii = matrices
    
    % get the angles in an array
    storeAngle.unNorm(:,ii) = angles.unNorm.(matStr{ii})(:);
    storeAngle.Phase(:,ii)  = angles.Phase.(matStr{ii})(:);
    
    % get the largest angle difference
    a = max( storeAngle.unNorm(:,ii) - storeAngle.Phase(:,ii));
    b = min( storeAngle.unNorm(:,ii) - storeAngle.Phase(:,ii));
    if abs(a) > abs(b)
        maxAngle(ii) = a;
    else
        maxAngle(ii) = b;
    end
end

% plotting time
figure
subplot(2,1,1)
hold on
plot( abs(storeAngle.unNorm(:)), 'o')
plot( abs(storeAngle.Phase(:)), 'x')
hold off
ylim([-2 100])
xlim([-50 8e3])
ylabel('Angle (deg)')
xlabel('Angle #')
legend('Unnormalized','Normalized', 'Location', 'East')
set( gca, 'ytick', [0 45 90])
FancyPlot(3)

subplot(2,1,2)
plot( matrices, maxAngle, 'o')
ylabel('Max difference (deg)')
xlabel('Matrix')
text( 130, 6.3, ['mean = ', num2str( round( mean(maxAngle),2)),...
    ' +/- ', num2str( round( std(maxAngle),3))])
xlim([-3 203])
ylim([-5 7])
set( gca, 'ytick', [-5 0 5])
FancyPlot(3)
end

function embedMatrix( U, matSize, matrices, tList, matStr, doIdeal, epsilon)
% do the matrix embedding idea
% 
% Input
%   U           Object with all matrices with (ideal) data
%   matSize     String indicating what matrix you want to embed
%               -'12x12' The full 12x12 matrix
%               -'12x6   The left half of the 12x12
%               -'6x6'   Only the 6x6 transmission matrix
%   matrices    Array of all matrices you want to check
%   tList       Object with all singular values from previous analysis
%   matStr      Cellarray with all matrix names as string
%   doIdeal     0   Process the experimental data
%               1   Process the ideal data
%               2   Process the ideal data + Gaussian noise
%   epsilon     Scaler, measure for Gaussian noise
%               Unew = U * ( 1 + epsilon * Haar_random)
%               typical value for experiment: ~0.1

% setup the correct numbers
if strcmp(matSize, '12x12')
    rowIn  = 1:12;
    colIn  = 1:12;
    rowSub = 7:12;
    colSub = 1:6;
    sizeS  = [12, 12];
    
elseif strcmp(matSize,'12x6')
    rowIn  = 1:12;
    colIn  = 1:6;
    rowSub = 7:12;
    colSub = 1:6;
    sizeS  = [12, 6];
else 
    % assume matSize == 6x6
    rowIn  = 7:12;
    colIn  = 1:6;
    rowSub = 1:6;
    colSub = 1:6;
    sizeS  = [6, 6];
end

% allocate memory
temp = zeros( 200, 6);

% loop through all matrices
counter = 0;
for ii = matrices
    % do the svd
    if doIdeal == 1
        [A,D,B] = svd(U.Ideal.(matStr{ii})(rowIn,colIn));
    elseif doIdeal == 2
        tempMat = U.Ideal.(matStr{ii}) .*( 1 + epsilon*getHaarRandmUnitary(12));
        [A,D,B] = svd( tempMat(rowIn,colIn));
    else
%         [A,D,B] = svd( U.unNorm.(matStr{ii})(rowIn,colIn));
        [A,D,B] = svd( U.Phase.(matStr{ii})(rowIn,colIn));

    end
    
    % compute the new unitary matrices 
    A2 = [A, zeros(sizeS(1)); zeros(sizeS(1)), eye(sizeS(1))];
    B2 = [B, zeros(sizeS(2)); zeros(sizeS(2)), eye(sizeS(2))];
    
    % normalize the singular values 
    % also count how often this occurs
    if max(max(D)) > 1
        Dn = (D / max(max(D)));
        counter = counter+1;
    else
        Dn = D;
    end
    
    % set all values correct by the L1 norm
    Di = D ./ max(D,1);             % rectangular 'diagonal' matrix
    D2 = [Dn, Di-Dn; Di-Dn, Dn];
    
    % set all values correct by the L2 norm
        % to implement
    
    
    % compute back the matrix
    M2 = A2*D2*B2';
    
    % get submatrix and the singular values
    T = M2(rowSub, colSub);
    
    if max(max(D)) > 1
        temp(ii,:) = ones(1,6)*-5;
    else
        temp(ii,:) = svd(T);
    end
    
end
disp( [num2str( round(counter/length(matrices)*100, 1)), '% had singular values > 1'])

figure
clf
subplot(2,1,1)
histogram( temp(:).^2, 0:0.01:1.5)
set( gca, 'yscale', 'log')
if doIdeal == 1
    title(['Matrix embedding, ', matSize, ' (ideal)'])
else
    title(['Matrix embedding, ', matSize, ' (exp)'])
end
FancyPlot(3)

subplot(2,1,2)
if doIdeal == 1
    histogram( tList.Ideal{1}(:)', 0:.01:1.5)
    title('Ideal (numerical)')
else
    histogram( tList.Phase{1}(:)', 0:.01:1.5)
    title('Normalized columns (exp)')
end
set( gca, 'yscale', 'log')
xlabel('Singular values \tau^2')
FancyPlot(3)



% figure
% plot( unitarity)
% ylabel('unitarity')
% xlabel('matrix')

end

function plotPowerDrift( Power, Rates, RatesC2, inputStrS)
figure
clf
plot(Power.Matrix_001, 'o', 'MarkerSize', 8)
FancyPlot(3)
set( gca, 'xtick', 1:11,...
    'xticklabel', inputStrS,...
    'TickLabelInterpreter', 'none',...
    'XTickLabelRotation', 45)
xlim([0 12])
ylim([45 50])
xlabel('Input configuration')
ylabel('Pump Power (mW)')


figure
clf
[ha, ~] = tight_subplot(2,1, 0, [.15 0.1], [.1 .1]);

axes(ha(1))
plot( sum( Rates.Matrix_001)/1e3, 'o', 'MarkerSize', 8)
xlim([0 13])
ylim([ 50 100])
set( gca, 'xtick', 1:12, 'xticklabel', '', 'ytick', 50:20:90)
ylabel('Counts (kHz)')
text( 6.5, 95, 'Uncorrected counts', 'HorizontalAlignment', 'center')
FancyPlot(3)

axes(ha(2))
plot( sum( RatesC2.Matrix_001)/1e3, 'o', 'MarkerSize', 8)
xlim([0 13])
ylim([ 50 100])
set( gca, 'xtick', 1:12, 'ytick', 50:20:90)
ylabel('Counts (kHz)')
xlabel('Input mode')
text( 6.5, 95, 'Corrected counts', 'HorizontalAlignment', 'center')
FancyPlot(3)
end


% row swapping functions
function HOM = swapHOM( HOM, swapRows)
% restructure the raw HOM counts in case you want to swap rows

% the order how is it stored right now
outputExp = nchoosek(1:12,2);

% now swap the order such that it matches with that of 'swapRows'
newOutput = zeros( size( outputExp));
indices = zeros(1,size(outputExp,1));
for ii = 1 : size( outputExp,1)
    
    % build up the new output combinations
    newOutput(ii,:) = sort([swapRows( outputExp(ii,1)), swapRows( outputExp(ii,2))]);
    
    % find the index of this output
    [~,indices(ii)] = ismember(newOutput(ii,:),outputExp,'rows');
end

HOM = HOM(:,indices);

end

function plotSwabbedElements( matrixPermut, nPlot, nRep, matrices, matStr)


for nn = matrices
pivotPos = [randperm(12)',randperm(12)'];


for jj = 1 : nRep
    pivot = pivotPos(1:nPlot,:);
    pivotPos(1:nPlot,:) = [];
    
    figure
    clf
    [ha, pos] = tight_subplot( nPlot, 2, 0, [.15 0.1], [.1 .1]);
    
    edgesAmp = linspace(0,2,26);
    edgesPhs = linspace(-pi,pi,26);
    
    for ii = 1 : nPlot
        axes(  ha((ii-1)*2+1))
        histogram( abs(matrixPermut.recon.(matStr{nn})( pivot(ii,1), pivot(ii,2), :)), edgesAmp)
        %         histogram( abs(matrixPermut.phase.(matStr{nn})( pivot(ii,1), pivot(ii,2), :)), edgesAmp)
        
        set( gca, 'xlim', [-.2 2.1],...
            'xtick', 0 :.5 : 4,...
            'xticklabel', '',...
            'ylim', [.5 500]*1.1,...
            'ytick', [1 10 100 ],...
            'yscale', 'log')
        FancyPlot(2)
        
        txtStr = ['U_{',num2str( pivot(ii,1)), ',', num2str( pivot(ii,2)),'}'];
        text( 1.8, 225, txtStr,'fontSize', 9)
        
        
        axes( ha(ii*2))
        histogram( angle(matrixPermut.recon.(matStr{nn})(  pivot(ii,1), pivot(ii,2), :)), edgesPhs)
        %         histogram( angle(matrixPermut.phase.(matStr{nn})(  pivot(ii,1), pivot(ii,2), :)), edgesPhs)
        
        set( gca, 'xlim', [-pi pi]*1.1,...
            'xtick', -pi : pi/2 : pi,...
            'xticklabel', '',...
            'ylim', [.5 500]*1.1,...
            'ytick', [1 10 100 ],...
            'yticklabel', '',...
            'yscale', 'log')
        FancyPlot(2)
        
        if ii == nPlot
            set( gca, 'xticklabel', {'-\pi', '-\pi /2', '0', '\pi /2', '\pi'})
            xlabel('Phase')
            
            axes(  ha((ii-1)*2+1))
            set( gca, 'xticklabel', {'0', '0.5', '1', '1.5', '2'})
            xlabel('Amplitude')
            
        end
    end
end
end
end

function plotSwabbedBestRows( savePermut, savePermut2, nBest, minF, HOMe, RatesC, data, matrices, matStr)


if ~isempty(matrices)
for nn = matrices
    matStr{nn}
    figure
    clf
    
    imagesc( max(savePermut2.(matStr{nn}), minF*ones(12))')
    set(gca,'YDir','normal',...
        'xtick',1:12,...
        'ytick',1:12)
    xlabel( '1st row')
    ylabel( '2nd row');
    c = colorbar();
    c.Limits = [minF 1];
    c.Label.String = 'Fidelity';
    
    % show 1 photon rates + HOM dip of the 10 best cases
    sortSavePermut = sortrows(savePermut.(matStr{nn}),1);
    markersStyle = {'o', 's', '^', 'x', 'p', 'h', '<', '>', '*', '+'};
    bla = nchoosek(1:12,2);
    % figure
    % clf
    % hold on
    for ii = [1 :nBest, 132]
        
        % to do: controleer of de HOM dip die ik zo hieruit haal ook
        % daadwerkelijk de juiste HOM dip is!
        
        R1 = sortSavePermut(end+1-ii,2);
        R2 = sortSavePermut(end+1-ii,3);
        ind = find( sum(ismember( bla, [R1 R2])') == 2);
        output =  getOutput( sort([R1 R2]), 1);
        Rtemp = RatesC.(matStr{nn})([R1,R2],1:2);
        
        %     disp( ['Row1 = ', num2str(R1)])
        %     disp( ['Row2 = ', num2str(R2)])
        disp( ['Fidelity = ', num2str(sortSavePermut(end+1-ii,1))])
        disp([ 'HOM V    = ', num2str(HOMe.(matStr{nn})(1,ind))])
        disp(['perm|U|  = ' num2str( (permanentRyser( Rtemp)))])
        %     disp( num2str(  Rtemp))
        disp(' ')
        
        
        %     plot( data.(matStr{nn}){1}.PositionList,...
        %         mean(data.(matStr{nn}){1}.Coincidences.(output)),...
        %         'Marker', markersStyle{mod(ii,length(markersStyle))+1})
        
    end

% legend
end
else

    
    summedF = zeros(12);
    for nn = 1 :200
        summedF = summedF + savePermut2.(matStr{nn});
    end
    
    figure
    clf
    imagesc( transpose( max(summedF/200, minF*ones(12))), [minF max(max(summedF/200))])
    set(gca,'YDir','normal',...
        'xtick',1:12,...
        'ytick',1:12)
    xlabel( '1st row')
    ylabel( '2nd row');
    c = colorbar();
    c.Limits = [minF 1];
    c.Label.String = 'Fidelity';
end
end

function plotSwabbedFidelities( nPermutations, Fswaps, matrices, matStr)

for nn = matrices
figure
clf

subplot(2,1,1)
colours = get(gca,'colororder');

hold on
plot( [0 nPermutations+50], mean(Fswaps.(matStr{nn})(4,:))*ones(1,2),...
            '--', 'Color', colours(2,:), 'HandleVisibility','off')
plot( [0 nPermutations+50], mean(Fswaps.(matStr{nn})(8,:))*ones(1,2),...
            '--', 'Color', colours(3,:), 'HandleVisibility','off')
plot( Fswaps.(matStr{nn})(2,:)', 'o', 'Color', colours(1,:), 'MarkerSize', 8, 'DisplayName', 'Ideal')
plot( Fswaps.(matStr{nn})(4,:)', 'x', 'Color', colours(2,:), 'MarkerSize', 8, 'DisplayName', 'Exp. rates')
plot( Fswaps.(matStr{nn})(8,:)', 's', 'Color', colours(3,:), 'MarkerSize', 8, 'DisplayName', 'Exp. all')


ylabel('Fidelity')
xlabel('Different row permutations')
title('Random permutations of the rows')
legend('Position', [0.782,0.565,0.158,0.164])

set( gca, 'xlim', [-2 nPermutations+52],...
    'xtick', 0:50:200,...
    'ylim', [0 1.1],...
    'ytick', 0:.5:1)

text( nPermutations+ 5, mean(Fswaps.(matStr{nn})(4,:))+0.1, num2str( round(mean(Fswaps.(matStr{nn})(4,:)),3),'%1.3f'), 'Color', colours(2,:))
text( nPermutations+25, mean(Fswaps.(matStr{nn})(8,:))+0.1, num2str( round(mean(Fswaps.(matStr{nn})(8,:)),3),'%1.3f'), 'Color', colours(3,:))

FancyPlot(3)


subplot(2,1,2)
stepSize = .02;
edges = 0:stepSize:1;
h1 = histcounts( Fswaps.(matStr{nn})(4,:), edges);
h2 = histcounts( Fswaps.(matStr{nn})(8,:), edges);

hold on
bar( edges(1:end-1)-stepSize/2, h1, 'faceColor', colours(2,:))
bar( edges(1:end-1)-stepSize/2, h2, 'faceColor', colours(3,:))
alpha(.5)

ylabel('Occurence')
xlabel('Fidelity')

set( gca, 'xlim', [0 1],...
    'xtick', 0 : .25 : 1,...
    'ylim', [0 30],...
    'ytick', 0 : 15 : 30)
FancyPlot(3)
end
end

function invPermut = getInversePermutation( permutation)
% gets the inverse permutation
% assumes that the permutation constains all numbers between 1 and N, 
% AND that the originial order is [1,2,3,...,N]

N = length( permutation);
indices = zeros(1,N);
for ii = 1 : N
    indices(ii) = find( permutation == ii);
end

invPermut = permutation( indices);
end



% misc test functions
function testIdealRandomPhases( Ut, matrices, matStr, doScrambleUnitary, scale)

Tscramble1 = [];
Tscramble2 = [];


for ii = matrices
    
    % get the (transmission) matrix
    if doScrambleUnitary == 0
        A = Ut.(matStr{ii}) .* exp( 1i* rand(12)*2*pi);
    elseif doScrambleUnitary == 1
        A = Ut.(matStr{ii}) .* (1 + scale*getHaarRandmUnitary(12));
    else
        A = Ut.(matStr{ii}) * (exp( 1i* rand(1,12)*2*pi) .* eye(12));
    end
    B = doPolarD(A);
    
    T3 = A( 7:12, 1:6);
    T4 = B( 7:12, 1:6);

    % get singular values
    t3 = svd(T3);
    t4 = svd(T4);
    Tscramble1 = [Tscramble1, t3.^2];
    Tscramble2 = [Tscramble2, t4.^2];
end

figure
subplot(2,1,1)
histogram(Tscramble1, linspace(0,2,101));
set( gca, 'yscale', 'log')
title('Ideal, random phases', 'fontweight', 'normal')
FancyPlot(3)
yRange = ylim;
yRange(1) = min([1, yRange(1)]);
ylim( yRange)

subplot(2,1,2)
[x,y] = fitDMPK( linspace(0,1,51), Tscramble2);
hold on
histogram(Tscramble2, linspace(0,1,51));
plot( x,y, 'r')
ylim( yRange)
title('Ideal, random phases + unitary constraint', 'fontweight', 'normal')
set( gca, 'yscale', 'log')
xlabel('singular values \tau^2')
FancyPlot(3)
end

function testIdealLossVariation( Ut, RatesC, matrices, matStr, scale)
Tloss1 = [];
Tloss2 = [];
for ii = matrices
    
    lossIn = sum( RatesC.(matStr{ii})')*scale;
    
    % get the (transmission) matrix
    A = Ut.(matStr{ii}) .* lossIn';
    B = doPolarD(A);
    
    T5 = A( 7:12, 1:6);
    T6 = B( 7:12, 1:6);
    
    % get singular values
    t5 = svd( T5);
    t6 = svd( T6);
    
    Tloss1 = [Tloss1, t5.^2];
    Tloss2 = [Tloss2, t6.^2];
end
figure
subplot(2,1,1)
histogram(Tloss1, linspace(0,2,101));
set( gca, 'yscale', 'log')
title('Ideal, with random input losses', 'fontweight', 'normal')
FancyPlot(3)
yRange = ylim;
yRange(1) = min([1, yRange(1)]);
ylim( yRange)

subplot(2,1,2)
[x,y] = fitDMPK( linspace(0,1,51), Tloss2);
hold on
histogram(Tloss2, linspace(0,1,51));
plot(x,y,'r')
ylim( yRange)

title('Ideal, with random input loss + unitary constraint', 'fontweight', 'normal')
set( gca, 'yscale', 'log')
xlabel('singular values \tau^2')  
FancyPlot(3)
end

function checkPumpPower( Power, matStr, matrices, inputStrS)

% loop through all matrices
for ii = matrices
    
    % loop through all input configurations
    for jj = 1 : 11
        % store the power
        inputPower{jj}(ii) = Power.(matStr{ii})(jj);
    end    
end

figure(1)
clf
[ha, ~] = tight_subplot(3,4, 0, [.15 0.1], [.1 .1]);
for jj = 1 : 12
    axes( ha(jj))
    
    if jj > 1
        bar( inputPower{jj-1})
    end
    
    set( gca, 'ylim', [50 56],...
        'xlim', [-5 max(matrices)+20],...
        'xtick', 0 : 100 : max(matrices),...
        'ytick', [50 52 54],...
        'box', 'on')
    
    if mod(jj,4) ~= 1
        set( gca, 'yticklabel', '')
    else
        ylabel('Pump power (mW)')
    end
    if jj < 8
        set( gca, 'xticklabel', '')
    else
        xlabel('Matrix')
    end
    if jj > 1
        text( 10, max(ylim)*.99, ['Input configuration ', inputStrS{jj-1}],...
            'Interpreter', 'none')
        text( 141, max(ylim)*.99, ['<P> = ', num2str(round(mean(inputPower{jj-1}),2), '%2.1f'),...
            ' +/- ', num2str(round(std( inputPower{jj-1}),2), '%2.2f')])
    end
    FancyPlot(3)
end
end

function checkTotalCounts( Rates, RatesC, matrices, matStr)
countsC = zeros(1,max(matrices));
counts  = zeros(1,max(matrices));

for ii = matrices
    
    countsC(1,ii) = sum(RatesC.(matStr{ii}), 'all');
    counts(1,ii)  = sum(Rates .(matStr{ii}), 'all');
end

figure
clf
[ha, ~] = tight_subplot(2,1, 0, [.15 0.1], [.1 .1]);

axes(ha(1))
hold on
bar( countsC/1E6)
plot( [0 200], mean( countsC/1E6)*ones(1,2), '--r')
hold off
xlabel('Matrix')
ylabel('Total counts (MHz)')
set( gca,     'xlim', [-2 202],...
    'ylim', [1 1.1],...
    'ytick', [1 1.05 round(mean(countsC/1e6),2) 1.1],...
    'xtick', [1, 50 : 50 : 200],...
    'xticklabel', '',...
    'yticklabel', {'1', '1.05', num2str(round(mean(countsC/1e6),2)), '1.1'})
FancyPlot(3)

axes(ha(2))
hold on
bar( counts/1E6)
plot( [0 200], mean( counts/1E6)*ones(1,2), '--r')
hold off
xlabel('Matrix')
ylabel('Total counts (MHz)')
set( gca,     'xlim', [-2 202],...
    'ylim', [0.5 1],...
    'ytick', [0.5 0.75 round(mean(counts/1e6),2)],...
    'xtick', [1, 50 : 50 : 200],...
    'xticklabel', {'1', '50', '100', '150', '200'},...
    'yticklabel', {'0.5', '0.75', num2str(round(mean(counts/1e6),2))})
FancyPlot(3)
end



% relative efficincies
function [relEff, cnts] = getDetectionLosses(Folder, filename, plotSingles, plotRel)
% Get the relative detection efficiencies (incl. outcoupling of the chip)
% for all modes
% Input
%   Folder      The folder where to find the data (incl. InputConfig_0x...)
%   filename    The basic filename (incl. wildcards *)
%   plotSingles If 1 or True, plot all data in 1 graph
%   plotRel     If 1 or True, plot the relative efficiencies
% 
% Output
%   relEff      Array of 1 x 12 with the relative detection efficiencies
%   cnts        total observed counts (optional output)

% get the filename and remove the output mode part so that I can later use
% it with indexing later
Folder0 = Folder;
files = dir([Folder0, filename]);
% files(1).name(end-5:end) = [];
% files(1).name(48:end) = [];

name0 = files(1).name;
% some settings
nModes = 12;

% in case you want a plot of the relative detection efficiencies
if plotSingles == 1
    figure
    clf
    maxY = 1;
end

% keep track of the maximal number of singles measured
relEffsingle = zeros(1,nModes);
relEff       = zeros(1,nModes);

% loop through all detectors and extract the counts
cnts1 = zeros(12);
cnts2 = zeros(12);
for ii = 1 : nModes
    
    
    % load in the data and get all swabian channel names
    loadName = [name0(1:end-6), num2str(ii,'%02.f'), '.mat'];
%     load( [Folder0, files(1).name, num2str(ii), '.mat.mat'])
    load( [Folder0, loadName])
    sNames = fieldnames( data.Singles);
    nNames = length(sNames);
    
    % loop through all swabian detection channel and get the counts
    singleCnt1 = zeros(1,nNames);
    output = zeros( 1, nNames);
    outputStr{nNames} = '';
    for jj = 1 : nNames
       singleCnt1(jj) = mean(data.Singles.(sNames{jj}));
       output(jj) = getOutput( sNames{jj});
       if mod(jj,2) == 0
           outputStr{jj} = num2str(jj);
       else
           outputStr{jj} = '';
       end
    end
    
    cNames = fieldnames( data.Coincidences);
    nNames = length( cNames);
    coinCnt1 = zeros(1, nNames);
    for jj = 1 : nNames
        coinCnt1(jj) = mean( data.Coincidences.(cNames{jj}));        
    end
    
    % sort the data in the order of the chip modes (before this: it was
    % sorted by the swabian channel order). Also, remove the trigger diode
    % (if present)
    ind = find( output == 0);
    output(ind) = [];
    singleCnt1(ind) = [];
    [~,ind] = sort( output);
    cnts1(ii,:) = singleCnt1(ind);
    cnts2(ii,:) = coinCnt1(ind);

    relEffsingle(ii) = max(cnts1(ii,:));
    relEff(ii)       = max(cnts2(ii,:));

        % in case a plot is desired: add the data to the plot
    if plotSingles == 1
        if max( cnts2(ii,:)) > maxY
            maxY = max( cnts2(ii,:));
        end
        subplot(3,4,ii)
        bar( cnts2(ii,:)/1000 )
        ylim([0 maxY/1000])
        title([ num2str(round( max(cnts2(ii,:))/sum(cnts2(ii,:)),3)*100, '%03.1f'), '%'],...
            'FontWeight', 'normal')
        set( gca, 'xtick', 1:nModes, 'xticklabels', '')
        if ii == 5
            ylabel('counts (kHz)')
        elseif ii > 8
            set( gca, 'xticklabels', outputStr)
        end
        FancyPlot( get( gcf, 'Number'),1)
    end
    
        
end

% normalize to get the relative efficiencies
cnts = relEffsingle;
relEffsingle = relEffsingle/max(relEffsingle);
relEff   = relEff  /max(relEff);


% numerical optimization to improve the results (only for coincidences)
% it is basically a gradient descent to minimize the standard devation of
% the reconstructed expected counts. The total photons should be equal for
% all measurements (that is, before losses)
nSteps = 5;
eta = 0.5 : 0.001 : 1;          % the eta's to try
testEta= relEff;                % initial condition
minSTD = zeros(1,length(eta));  

% do the optimization a few times
for nn = 1 : nSteps
    
    % try all output modes
    for cc = 1 : 12
        
        % do the gradient descent
        for ii = 1 : length( eta)
            % select the new test value
            testEta(cc) = eta(ii);
            gain = 1 ./testEta';

            % reconstruct how the counts would have been
            counts = sum( gain .*cnts2,2);
            minSTD(ii) = std( counts);
        end
        
        % now find back the eta that worked best
        ind = find( minSTD == min( minSTD));
        testEta(cc) = eta(ind(1));
    end
end

% now store the result
relEff = testEta/max(testEta);


% plot the relative counts
if plotRel == 1
    figure
    bar( 1:12, [relEffsingle; relEff])
    xlabel('Chip output mode')
    ylabel('Relative efficiencies')
    title(['Input = ', Folder(end-5:end-1)], 'Interpreter', 'None')
    ylim([0 1.05])
    legend('Singles','Trigger coincidence')
    FancyPlot( get( gcf, 'Number'),1)
end



end

function [input,output] = getRelativeTransmission()
% returns the relative transmission coefficients as given by Quix for the
% incoupling and outcoupling
% Output
%   input   1x12 array with the relative incoupling losses
%   output  1x12 array with the relative outcoupling losses

Folder = 'M:\Data\DMPK on chip\Experiment\chip transmission efficiencies\';
FileName1 = 'Quix_12x12_Camel_Losses_Input.mat';
FileName2 = 'Quix_12x12_Camel_Losses_Output.mat';

a = load( [Folder, FileName1]);
b = load( [Folder, FileName2]);

input = a.LossCol(1,:)/max(a.LossCol(1,:));
output= b.LossRow(1,:)/max(b.LossRow(1,:));
end



% misc.
function U = rephase(U)
% Sets the phases of the first column and row to zero by left and right
% multiplication of diagonal matrices.
% Input
%   U   nxn unitary matrix
% Output
%   U   nxn unitary matrix with the first rol/col no phases

% Input validation
s = size(U);
if length(s) ~= 2
    error('U must be 2 dimensional matrix')
end
if s(1) ~= s(2)
    error('U must be a square matrix')
end

% Rephase row 1
D1 = diag(exp(-1i*angle(U(1,:))));
U = U*D1;

% Rephase column 1
D2 = diag(exp(-1i*angle(U(:,1))));
U = D2*U;

% guarantee a positive phase @ element (2,2)
% don't do this as this is problematic. It will make the U(2,2) angle
% positive, however the top row is not angle = 0 anymore
% if angle(U(2,2))<0
%     U = abs(U) * exp( -1i* angle(U));
% end
end

function [Fidelity,B] = getFidelity( A, B)
% computes the fidelity as defined in the Burgwal Imperfections paper
% (Optics Express 2017)
% note that the phase sign can be ambigious, so if the fidelity is too
% small, simply try to flip the phases
% Input
%   A           nxn unitary matrix
%   B           nxn unitary matrix
% Ouput
%   Fidelity    The fidelity between the matrices


if size(A,1) ~= size(A,2)
    norm = sqrt(trace(abs(A*A'))*trace(abs(B*B')) );
    m = norm;
else
    m = length(A);
end
F1 = 1 / m * trace( abs(A'*B));

% in a bad outcome: try flipping the phases
R2 = abs(B) .*exp(-1i*angle(B));
F2 = 1 / m * trace( abs(A'*B));

% take the best value
if F1 > F2
    Fidelity = F1;
else
    Fidelity = F2;
    B = R2;
end

end

function R = doPolarD( R)
    margin = 1E-3;
    nn = length(R);
    if abs(max(max(R*R' - eye(nn)))) > margin
%         disp('Non unitary matrix found: do a polar decomposition')
        [A,~,B] = svd(R);
        R = A*B';
        
        % reset the first row/col to phase 0
        R = rephase(R);
    end
end

function B = optimizeInputPhase( A, B)
% optimize the global phases to maximize the fidelity
% necessary because of the stupid burgwal definition, which is strangely
% dependent on the global phase
% A is target matrix
% B is experimental matrix which you want to optimize

% since it is a periodic function (phase), I bound the results between -pi
% and pi and do a multistart solver to prevent local optima close to the
% bounds
opts = optimoptions( @fmincon, 'Algorithm', 'sqp');
bndLow = -pi*1*ones(12,1);
bndHigh= -bndLow;
x0     = angle(A(:,1))-angle(B(:,1));
fun = @(D) abs(1-getFidelity(A,exp(1i*D).*B));

problem = createOptimProblem( 'fmincon', 'objective',...
    fun, 'x0', x0, 'lb', bndLow, 'ub', bndHigh,'options',opts);
ms = MultiStart;
[D,f] = run( ms, problem, 20);

% D = fmincon( fun, x0, [], [], [], [], bndLow, bndHigh);

% rescale the exp matrix B with the new input phases
B = exp(1i*D).* B;
end

function B = optimizeIOPhases( A, B)
% optimize the global phases at the individual input and output modes to 
% maximize the fidelity necessary because of the stupid burgwal definition, 
% which is strangely dependent on the I/O phases

% Generalization of "optimizeInputPhase" above.

% A is target matrix
% B is experimental matrix which you want to optimize

m = length(A);
% m1 = size(A,1);
% m2 = size(A,2);

%array with the phases
% PH = zeros(m1 + m2);
PH = zeros(2*m);

opts = optimoptions( @fmincon, 'Algorithm', 'sqp');
% bndLow = -pi*1*ones(m1+m2,1);
bndLow = -pi*1*ones(2*m,1);
bndHigh= -bndLow;
x0     = [angle(A(:,1)) angle(A(:,2))];
% fun = @(PH) abs(1-getFidelity(A  ,  diag(exp(1i*PH(1:m1)))*B*diag(exp(1i*PH(m1+1:m1+m2)))));
fun = @(PH) abs(1-getFidelity(A  ,  diag(exp(1i*PH(1:m)))*B*diag(exp(1i*PH(m+1:2*m)))));

problem = createOptimProblem( 'fmincon', 'objective',...
    fun, 'x0', x0, 'lb', bndLow, 'ub', bndHigh,'options',opts);
ms = MultiStart;
[D,f] = run( ms, problem, 20);

% D = fmincon( fun, x0, [], [], [], [], bndLow, bndHigh);

% rescale the exp matrix B with the new input phases
% B = diag(exp(1i*D(1:m1)))*B*diag(exp(1i*D(m1+1:m1+m2)));
B = diag(exp(1i*D(1:m)))*B*diag(exp(1i*D(m+1:2*m)));
end
